# -*- coding: UTF-8 -*-
# vim: fdm=marker et ts=4 sw=4
__revision__ = '$Id$'

# Copyright (c) 2008 Vasco Nunes, Piotr Ożarowski
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

import logging
from copy import deepcopy
from gettext import gettext as _

from gi.repository import Gtk
from sqlalchemy import select

import db
import sql
from gutils import info, warning

log = logging.getLogger("Griffith")

__conditions = { # default
    'loaned'          : None,  # None, True, False
    'seen'            : None,  # None, True, False
    'collections'     : set(), # list of collection_ids (search for movies in these collections only)
    'no_collections'  : set(), # list of collection_ids (search for movies outside these collections)
    'volumes'         : set(), # list of volume_ids     (search for movies in these volumes only)
    'no_volumes'      : set(), # list of volume_ids     (search for movies outside these volumes)
    'tags'            : set(), # list of tag_ids        (search for movies with these tags)
    'no_tags'         : set(), # list of tag_ids        (search for movies without these tags)
    'required_tags'   : set(), # like in tags, but movie must contain all listed tags
    'loaned_to'       : set(), # list of person_ids     (search for movies loaned to these people)
    'loan_history'    : set(), # list of person_ids     (search for movies which were loaned by these people)
    'sort_by'         : set(("number",)), # "number DESC"
    'equals'          : {}, # {column1: set(value1, value2), column2: set(value3)}
    'equals_n'        : {}, # see above
    'startswith'      : {}, # see above
    'startswith_n'    : {}, # see above
    'contains'        : {}, # see above
    'contains_n'      : {}, # see above
    'like'            : {}, # see above
    'like_n'          : {}, # see above
    }

QUERY_FIELDS = ('title', 'o_title', 'director', 'plot', 'cast', 'notes', 'number',
                'runtime', 'year', 'screenplay', 'cameraman', 'country', 'genre',
                'studio', 'classification', 'o_site', 'site', 'trailer')
QUERY_COMMANDS = ('equals', 'equals_n', 'startswith', 'startswith_n', 'contains',
                  'contains_n', 'like', 'like_n')
QUERY_COMMAND_NAMES = {
    'equals'      : _('is equal to'),
    'equals_n'    : _('is not equal to'),
    'startswith'  : _('starts with'),
    'startswith_n': _("doesn't start with"),
    'contains'    : _('contains'),
    'contains_n'  : _("doesn't contain"),
    'like'        : _('like'),
    'like_n'      : _('not like')}

# widgets -----------------------------------------------------{{{


def show_window(self):
    if self.widgets['advfilter']['window'].props.visible:
        self.widgets['advfilter']['window'].present()
        return True
    initialize(self.widgets['advfilter'], self.db, self.field_names)

    if getattr(self, '_search_conditions', None):
        set_conditions(self.widgets['advfilter'], self._search_conditions, self.field_names)

    self.widgets['advfilter']['window'].show()

    return True


def hide_window(self):
    widgets = self.widgets['advfilter']
    widgets['window'].hide()

    # clear the window
    for i in widgets['tags_vbox'].get_children():
        i.destroy()
    for i in widgets['volumes_vbox'].get_children():
        i.destroy()
    for i in widgets['collections_vbox'].get_children():
        i.destroy()
    for i in widgets['loans_vbox'].get_children():
        i.destroy()
    for i in widgets['dynamic_vbox'].get_children():
        i.destroy()

    from initialize import fill_advfilter_combo
    fill_advfilter_combo(self)

    return True


def _fill_container(container, items, options, id_name):
    for item in items:
        hbox = Gtk.HBox()

        label_id = Gtk.Label()
        label_id.set_text(str(getattr(item, id_name)))
        hbox.pack_start(label_id, False, True, 0)

        widget = Gtk.RadioButton.new_with_label(None, options[0]) # first widget
        hbox.pack_start(widget, False, True, 4)
        for option in options[1:]: # create rest of the widgets, use first one as a group
            next_widget = Gtk.RadioButton.new_with_label_from_widget(widget, option)
            hbox.pack_start(next_widget, False, True, 4)

        label = Gtk.Label()
        label.set_text(item.name)
        hbox.pack_start(label, False, True, 16)

        hbox.show_all()
        label_id.hide()
        container.pack_start(hbox, True, True, 0)


def initialize(widgets, gsql, field_names):
    # tags
    items = gsql.session.query(db.Tag.tag_id, db.Tag.name).all()
    if len(items):
        options = (_('ignore'), _('with'), _('without'), _('require'))
        _fill_container(widgets["tags_vbox"], items, options, 'tag_id')
        widgets["tags_frame"].show()
    else:
        widgets["tags_frame"].hide()

    # volumes
    items = gsql.session.query(db.Volume.volume_id, db.Volume.name).all()
    options = (_('ignore'), _('in'), _('not in'))
    if len(items):
        _fill_container(widgets["volumes_vbox"], items, options, 'volume_id')
        widgets["volumes_frame"].show()
    else:
        widgets["volumes_frame"].hide()

    # collections
    items = gsql.session.query(db.Collection.collection_id, db.Collection.name).all()
    if len(items):
        # use volume's options
        _fill_container(widgets["collections_vbox"], items, options, 'collection_id')
        widgets["collections_frame"].show()
    else:
        widgets["collections_frame"].hide()

    # loans
    items = gsql.session.query(db.Person.person_id, db.Person.name).all()
    if len(items):
        options = (_('ignore'), _('loaned to '), _('loan history'))
        _fill_container(widgets["loans_vbox"], items, options, 'person_id')
        widgets["loans_frame"].show()
    else:
        widgets["loans_frame"].hide()

    widgets['cb_name'].get_model().clear()
    search_filters = gsql.session.query(db.Filter.name).order_by(db.Filter.name).all()
    for filter_ in search_filters:
        widgets['cb_name'].append_text(filter_[0])
    return True


def add_query_widget(container, field_names, sel_qf='title', sel_comm='contains', text=''):
    hbox = Gtk.HBox()

    cb = Gtk.ComboBoxText()
    select = 0
    for i, field in enumerate(QUERY_FIELDS):
        if sel_qf == field:
            select = i
        cb.append_text(field_names[field])
    cb.set_active(select)

    action_cb = Gtk.ComboBoxText()
    select = 0
    for i, command in enumerate(QUERY_COMMANDS):
        if sel_comm == command:
            select = i
        action_cb.append_text(QUERY_COMMAND_NAMES[command])
    action_cb.set_active(select)

    entry = Gtk.Entry()
    entry.set_text(text)

    button = Gtk.Button(stock=Gtk.STOCK_DELETE)
    button.connect("clicked", lambda w: hbox.destroy())

    hbox.pack_start(cb, False, True, 0)
    hbox.pack_start(action_cb, False, True, 0)
    hbox.pack_start(entry, True, True, 8)
    hbox.pack_start(button, False, False, 0)
    hbox.show_all()

    container.pack_start(hbox, True, True, 0)


def set_conditions(widgets, cond, field_names): #{{{
    widgets['name'].set_text('')

    # delete old widgets
    for i in widgets['dynamic_vbox'].get_children():
        i.destroy()

    if cond["seen"] is None:
        widgets["rb_seen"].set_active(True)
    elif cond["seen"] is True:
        widgets["rb_seen_only"].set_active(True)
    elif cond["seen"] is False:
        widgets["rb_seen_only_n"].set_active(True)

    if cond["loaned"] is None:
        widgets["rb_loaned"].set_active(True)
    elif cond["loaned"] is True:
        widgets["rb_loaned_only"].set_active(True)
    elif cond["loaned"] is False:
        widgets["rb_loaned_only_n"].set_active(True)

    for hbox in widgets["tags_vbox"]:
        hbox_items = hbox.get_children()
        id_ = int(hbox_items[0].get_text())
        if id_ in cond['tags']:
            hbox_items[2].set_active(True)
        elif id_ in cond['no_tags']:
            hbox_items[3].set_active(True)
        elif id_ in cond['required_tags']:
            hbox_items[4].set_active(True)
        else:
            hbox_items[1].set_active(True)

    for hbox in widgets["volumes_vbox"]:
        hbox_items = hbox.get_children()
        id_ = int(hbox_items[0].get_text())
        if id_ in cond['volumes']:
            hbox_items[2].set_active(True)
        elif id_ in cond['no_volumes']:
            hbox_items[3].set_active(True)
        else:
            hbox_items[1].set_active(True)

    for hbox in widgets["collections_vbox"]:
        hbox_items = hbox.get_children()
        id_ = int(hbox_items[0].get_text())
        if id_ in cond['collections']:
            hbox_items[2].set_active(True)
        elif id_ in cond['no_collections']:
            hbox_items[3].set_active(True)
        else:
            hbox_items[1].set_active(True)

    for hbox in widgets["loans_vbox"]:
        hbox_items = hbox.get_children()
        id_ = int(hbox_items[0].get_text())
        if id_ in cond['loaned_to']:
            hbox_items[2].set_active(True)
        elif id_ in cond['loan_history']:
            hbox_items[3].set_active(True)
        else:
            hbox_items[1].set_active(True)

    if not cond["equals"] and not cond["startswith"] and not cond["contains"] and not cond["like"]:
        for i in widgets['dynamic_vbox'].get_children():
            i.destroy()
        add_query_widget(widgets['dynamic_vbox'], field_names)
    else:
        vbox = widgets['dynamic_vbox']
        for rule in ('equals', 'startswith', 'contains', 'like'):
            for field in cond[rule]:
                for text in cond[rule][field]:
                    add_query_widget(vbox, field_names, field, rule, text)
    return True

    #}}}


def get_conditions(widgets): #{{{
    cond = get_def_conditions()

    if widgets["rb_seen"].get_active():
        cond["seen"] = None
    elif widgets["rb_seen_only"].get_active():
        cond["seen"] = True
    elif widgets["rb_seen_only_n"].get_active():
        cond["seen"] = False

    if widgets["rb_loaned"].get_active():
        cond["loaned"] = None
    elif widgets["rb_loaned_only"].get_active():
        cond["loaned"] = True
    elif widgets["rb_loaned_only_n"].get_active():
        cond["loaned"] = False

    for hbox in widgets["tags_vbox"]:
        hbox_items = hbox.get_children()
        if hbox_items[2].get_active():
            cond["tags"].add(int(hbox_items[0].get_label()))
        elif hbox_items[3].get_active():
            cond["no_tags"].add(int(hbox_items[0].get_label()))
        elif hbox_items[4].get_active():
            cond["required_tags"].add(int(hbox_items[0].get_label()))

    for hbox in widgets["volumes_vbox"]:
        hbox_items = hbox.get_children()
        if hbox_items[2].get_active():
            cond["volumes"].add(int(hbox_items[0].get_label()))
        elif hbox_items[3].get_active():
            cond["no_volumes"].add(int(hbox_items[0].get_label()))

    for hbox in widgets["collections_vbox"]:
        hbox_items = hbox.get_children()
        if hbox_items[2].get_active():
            cond["collections"].add(int(hbox_items[0].get_label()))
        elif hbox_items[3].get_active():
            cond["no_collections"].add(int(hbox_items[0].get_label()))

    for hbox in widgets["loans_vbox"]:
        hbox_items = hbox.get_children()
        if hbox_items[2].get_active():
            cond["loaned_to"].add(int(hbox_items[0].get_label()))
        elif hbox_items[3].get_active():
            cond["loan_history"].add(int(hbox_items[0].get_label()))

    for hbox in widgets["dynamic_vbox"]:
        hbox_items = hbox.get_children()

        entry = hbox_items[2].get_text().strip()
        if not entry: # ignore if it's empty
            continue

        field = hbox_items[0].get_active()
        if 0 > field or field > len(QUERY_FIELDS):
            continue
        else:
            field = QUERY_FIELDS[field]

            command = hbox_items[1].get_active()
            if 0 > command or command > len(QUERY_COMMANDS):
                continue
            else:
                command = QUERY_COMMANDS[command]

        cond[command].setdefault(field, set()).add(entry)

    return cond # }}}


def save(gsql, widgets):
    """saves search conditions from current filter window"""

    cond = get_conditions(widgets)
    name = widgets['cb_name'].get_active_text()
    if not name:
        log.debug("search rule name is empty")
        info(_("Name is empty"), widgets['window'])
        return False

    session = gsql.Session()
    filter_ = session.query(db.Filter).filter_by(name=name).first()
    if filter_:
        filter_.data = cond
        isnew = False
    else:
        filter_ = db.Filter(name=name, data=cond)
        isnew = True
    session.add(filter_)
    try:
        session.commit()
    except Exception as e:
        session.rollback()
        log.warn(e)
        warning(_("Cannot save search conditions"), widgets['window'])
        return False
    else:
        if isnew:
            widgets['cb_name'].append_text(name)
    info(_("Search conditions saved"), widgets['window'])


def load(gsql, widgets, field_names):
    name = widgets['cb_name'].get_active_text()
    if not name:
        log.debug("search rule name is empty")
        return False
    cond = gsql.session.query(db.Filter).filter_by(name=name).first()
    if cond:
        return set_conditions(widgets, cond, field_names)
    else:
        return False


def delete(gsql, widgets):
    """deletes the current selected filter"""

    name = widgets['cb_name'].get_active_text()
    if not name:
        log.debug("search rule name is empty")
        info(_("Name is empty"), widgets['window'])
        return False

    session = gsql.Session()
    filter = session.query(db.Filter).filter_by(name=name).first()
    if filter:
        session.delete(filter)
        try:
            session.commit()
        except Exception:
            session.rollback()
            log.exception('')
            warning(_("Cannot delete search conditions"), widgets['window'])
            return False
        else:
            widgets['cb_name'].remove_text(widgets['cb_name'].get_active())
        info(_("Search conditions deleted"), widgets['window'])
    widgets['name'].set_text('')

#}}}

# database related --------------------------------------------{{{


def get_def_conditions():
    return deepcopy(__conditions)


def get_select_columns(config): # {{{
    # TODO: get them from config
    columns_to_select = [db.Movie.number,
        db.Movie.o_title, db.Movie.title,
        db.Movie.director, db.Movie.poster_md5,
        db.Movie.genre, db.Movie.seen,
        db.Movie.year, db.Movie.runtime,
        db.Movie.rating, db.Movie.created, db.Movie.updated]
    return columns_to_select # }}}


def create_select_query(self, columns, conditions, query):
    if not conditions:
        conditions = get_conditions(self.widgets["advfilter"])

    if query is None: # initial query not set so create one
        if not columns:
            columns = get_select_columns(self.config)
        query = select(*columns)

    self._search_conditions = conditions # save for later
    # TODO: remove after debugging:
#    from pprint import pprint
#    pprint(conditions)

    return sql.update_whereclause(query, conditions)

#}}}



# -*- coding: UTF-8 -*-
# vim: fdm=marker
__revision__ = '$Id$'
# XXX: database format version, remember to increase after changing data
# structures
__version__ = 6

# Copyright © 2009 Piotr Ożarowski
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

from sqlalchemy import MetaData, func, select, and_
from sqlalchemy.orm import deferred, column_property, synonym

metadata = MetaData()
from . import tables  # *after* metadata initialization
#from ._objects import *
from ._objects import AChannel, ACodec, Lang, Medium, Ratio, SubFormat, Tag, \
    VCodec, Filter, Collection, Volume, Loan, Person, Poster, Configuration, \
    MovieLang, MovieTag
from ._movie import Movie

try:
    from sqlalchemy.orm import relationship, registry

    mapper_registry = registry()

    mapper_registry.map_imperatively(Configuration, tables.configuration)
    mapper_registry.map_imperatively(Volume, tables.volumes, properties={
        'loaned': synonym('_loaned', map_column=True),
        'movies': relationship(Movie, backref='volume',
                               order_by=tables.volumes.c.name)
        })
    mapper_registry.map_imperatively(
        Collection, tables.collections,
        properties={
            'loaned': synonym('_loaned', map_column=True),
            'movies': relationship(Movie, backref='collection',
                                   order_by=tables.collections.c.name)})
    mapper_registry.map_imperatively(Medium, tables.media, properties={
        'movies': relationship(Movie, backref='medium')})
    mapper_registry.map_imperatively(Ratio, tables.ratios, properties={
        'movies': relationship(Movie, backref='ratio')})
    mapper_registry.map_imperatively(VCodec, tables.vcodecs, properties={
        'movies': relationship(Movie, backref='vcodec')})
    mapper_registry.map_imperatively(Person, tables.people, properties={
        'loans': relationship(Loan, backref='person',
                              cascade='all, delete-orphan'),
        'loaned_movies_count': column_property(
            select(func.count(tables.loans.c.loan_id))
            .where(
                and_(
                    tables.people.c.person_id == tables.loans.c.person_id,
                    tables.loans.c.return_date.is_(None),
                )
            )
            .correlate_except(tables.loans)
            .scalar_subquery(),
            deferred=True,
        ),
        'returned_movies_count': column_property(
            select(func.count(tables.loans.c.loan_id))
            .where(
                and_(
                    tables.people.c.person_id == tables.loans.c.person_id,
                    tables.loans.c.return_date.is_not(None),
                )
            )
            .correlate_except(tables.loans)
            .scalar_subquery(),
            deferred=True,
        )
        })
    mapper_registry.map_imperatively(MovieLang, tables.movie_lang,
                                     primary_key=[tables.movie_lang.c.ml_id],
                                     properties={
                                         'movie': relationship(Movie),
                                         'language': relationship(Lang),
                                         'achannel': relationship(AChannel),
                                         'acodec': relationship(ACodec),
                                         'subformat': relationship(SubFormat)
                                         })
    mapper_registry.map_imperatively(ACodec, tables.acodecs, properties={
        'movielangs': relationship(MovieLang)})
    mapper_registry.map_imperatively(AChannel, tables.achannels, properties={
        'movielangs': relationship(MovieLang)})
    mapper_registry.map_imperatively(SubFormat, tables.subformats, properties={
        'movielangs': relationship(MovieLang)})
    mapper_registry.map_imperatively(Lang, tables.languages, properties={
        'movielangs': relationship(MovieLang)})
    mapper_registry.map_imperatively(MovieTag, tables.movie_tag)
    mapper_registry.map_imperatively(
        Tag, tables.tags,
        properties={'movietags': relationship(MovieTag, backref='tag')})
    mapper_registry.map_imperatively(Loan, tables.loans, properties={
        'volume': relationship(Volume),
        'collection': relationship(Collection)})
    mapper_registry.map_imperatively(Movie, tables.movies, properties={
        'loans': relationship(Loan, backref='movie',
                              cascade='all, delete-orphan'),
        # 'tags': relation(Tag, cascade='all, delete-orphan',
        # secondary=movie_tag,
        'tags': relationship(
            Tag, secondary=tables.movie_tag,
            primaryjoin=tables.movies.c.movie_id == tables.movie_tag.c.movie_id,
            secondaryjoin=tables.movie_tag.c.tag_id == tables.tags.c.tag_id),
        'languages': relationship(MovieLang, cascade='all, delete-orphan')
        })
    mapper_registry.map_imperatively(Poster, tables.posters, properties={
        'movies': relationship(Movie),
        'data': deferred(tables.posters.c.data)})
    mapper_registry.map_imperatively(Filter, tables.filters)


except Exception as ex:
    log.error(
        f"[E010]Caught too broad try-exception {type(ex).__name__} "
        f"with arguments '{ex.args}'")
    from sqlalchemy.orm import relation, mapper

    mapper(Configuration, tables.configuration)
    mapper(Volume, tables.volumes, order_by=tables.volumes.c.name, properties={
        'loaned': synonym('_loaned', map_column=True),
        'movies': relation(Movie, backref='volume')})
    mapper(Collection, tables.collections, order_by=tables.collections.c.name,
           properties={
               'loaned': synonym('_loaned', map_column=True),
               'movies': relation(Movie, backref='collection')
               })
    mapper(Medium, tables.media, properties={
        'movies': relation(Movie, backref='medium')})
    mapper(Ratio, tables.ratios, properties={
        'movies': relation(Movie, backref='ratio')})
    mapper(VCodec, tables.vcodecs, properties={
        'movies': relation(Movie, backref='vcodec')})
    mapper(Person, tables.people, properties={
        'loans': relation(Loan, backref='person',
                          cascade='all, delete-orphan'),
        'loaned_movies_count': column_property(
            select(func.count(tables.loans.c.loan_id))
            .where(
                and_(
                    tables.people.c.person_id == tables.loans.c.person_id,
                    tables.loans.c.return_date.is_(None),
                )
            )
            .correlate_except(tables.loans)
            .scalar_subquery(),
            deferred=True,
        ),
        'returned_movies_count': column_property(
            select(func.count(tables.loans.c.loan_id))
            .where(
                and_(
                    tables.people.c.person_id == tables.loans.c.person_id,
                    tables.loans.c.return_date.is_not(None),
                )
            )
            .correlate_except(tables.loans)
            .scalar_subquery(),
            deferred=True,
        ),
        })
    mapper(MovieLang, tables.movie_lang,
           primary_key=[tables.movie_lang.c.ml_id], properties={
            'movie': relation(Movie),
            'language': relation(Lang),
            'achannel': relation(AChannel),
            'acodec': relation(ACodec),
            'subformat': relation(SubFormat)
            })
    mapper(ACodec, tables.acodecs, properties={
        'movielangs': relation(MovieLang)})
    mapper(AChannel, tables.achannels, properties={
        'movielangs': relation(MovieLang)})
    mapper(SubFormat, tables.subformats, properties={
        'movielangs': relation(MovieLang)})
    mapper(Lang, tables.languages, properties={
        'movielangs': relation(MovieLang)})
    mapper(MovieTag, tables.movie_tag)
    mapper(Tag, tables.tags,
           properties={'movietags': relation(MovieTag, backref='tag')})
    mapper(Loan, tables.loans, properties={
        'volume': relation(Volume),
        'collection': relation(Collection)})
    mapper(Movie, tables.movies, order_by=tables.movies.c.number, properties={
        'loans': relation(Loan, backref='movie', cascade='all, delete-orphan'),
        # 'tags': relation(Tag, cascade='all, delete-orphan',
        # secondary=movie_tag,
        'tags': relation(
            Tag, secondary=tables.movie_tag,
            primaryjoin=tables.movies.c.movie_id == tables.movie_tag.c.movie_id,
            secondaryjoin=tables.movie_tag.c.tag_id == tables.tags.c.tag_id),
        'languages': relation(MovieLang, cascade='all, delete-orphan')
        })
    mapper(Poster, tables.posters, properties={
        'movies': relation(Movie),
        'data': deferred(tables.posters.c.data)})
    mapper(Filter, tables.filters)



# -*- coding: UTF-8 -*-

__revision__ = '$Id$'

# Copyright © 2009 Piotr Ożarowski
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published byp
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

import logging

from sqlalchemy.sql import update, select

from db.tables import movies as movies_table
from gutils import question
from plugins.extensions import GriffithExtensionBase as Base
from sql import update_whereclause
from quick_filter import change_filter_update_whereclause
from gettext import gettext as _

log = logging.getLogger('Griffith')


class GriffithExtension(Base):
    name = 'Mark as seen'
    description = _('Marks all currently filtered movies as seen')
    author = 'Piotr Ożarowski'
    email = 'piotr@griffith.cc'
    version = 0.1
    api = 1
    enabled = False  # disabled by default

    toolbar_icon = 'ge_mark_seen.png'

    def toolbar_icon_clicked(self, widget, movie):
        if question(_('Are you sure you want to update %d movies?') % self.app.total):
            session = self.db.Session()

            query = select(movies_table.c.movie_id)
            # add conditions from simple filter
            change_filter_update_whereclause(self.app, query)
            # add conditions from adv filter
            query = update_whereclause(query, self.app._search_conditions)
            movie_ids = []
            for movie_entry in session.execute(query):
                movie_ids.append(movie_entry.movie_id)

            update_query = update(movies_table).values(seen=True)
            update_query = update_query.where(movies_table.c.movie_id.in_(movie_ids))
            log.debug(update_query)
            session.execute(update_query)
            session.commit()

            self.app.populate_treeview()  # update seen widget in the list



# -*- coding: UTF-8 -*-
__revision__ = '$Id$'

# Copyright © 2009-2011 Piotr Ożarowski
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

import logging

from sqlalchemy import and_, or_
from sqlalchemy.orm import object_session

from . import tables
from ._objects import Loan, DBTable

log = logging.getLogger('Griffith')

res_aliases = {
    (2560, 1600): ('QSXGA',),
    (2048, 1536): ('QXGA',),
    (1920, 1200): ('WUXGA',),
    (1920, 1080): ('1080p', 'HD 1080', '1080'),
    (1920, 540): ('1080i',),
    (1680, 1050): ('WSXGA+',),
    (1600, 1200): ('UXGA',),
    (1400, 1050): ('SXGA+',),
    # (1440, 960): ('',),
    # (1280, 960): ('',),
    # (1280, 854): ('',),
    (1280, 720): ('720p', 'HD 720', '720'),
    (1280, 360): ('720i',),
    (1280, 1024): ('SXGA',),
    # (1152, 768): ('',),
    (1024, 768): ('XGA',),
    (854, 480): ('WVGA',),
    (800, 600): ('SVGA',),
    (768, 576): ('PAL',),
    (720, 480): ('NTSC',),
    (640, 480): ('VGA',),
    (320, 240): ('QVGA',),
    (320, 200): ('CGA',),
    # some kine of "virtual" resolutions
    (1, 1): ('4:3 Fullscreen',),
    (1, 2): ('Widescreen',),
    (1, 3): ('Anamorphic Widescreen',)
    }
res_alias_res = {}
for res, aliases in res_aliases.items():
    for alias in aliases:
        res_alias_res[alias.upper()] = res
del aliases, alias, res


class Movie(DBTable):

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.loaned = False

    def _set_resolution(self, res_string):
        if not res_string:  # clear resulution field
            self.width = None
            self.height = None
        elif res_string.upper() in res_alias_res:
            self.width, self.height = res_alias_res[res_string.upper()]
        else:
            try:
                if 'x' in res_string:
                    self.width, self.height = (
                        list(map(int, res_string.lower().split('x'))))
                else:
                    self.width, self.height = (
                        list(map(int, res_string.lower().split())))
            except Exception as e:
                log.warning('wrong resolution name: %s', e)
                raise ValueError(r'Use standard resolution name or \d+x\d+')

    def _get_resolution(self):
        if not self.width or not self.height:
            return None
        resolution = (self.width, self.height)
        if resolution in res_aliases:
            return res_aliases[resolution][0]
        else:
            res_string = "%dx%d" % resolution
            return res_string

    resolution = property(_get_resolution, _set_resolution)

    def __repr__(self):
        return "<Movie:%s (number=%s)>" % (self.movie_id, self.number)

    def __contains__(self, name):
        media_properties = (
            'volume', 'collection', 'medium', 'vcodec', 'loans', 'tags',
            'languages', 'lectors', 'dubbings', 'subtitles', 'resolution')
        if name in media_properties:
            return True
        else:
            return name in tables.movies.columns

    def __getitem__(self, name):
        if hasattr(self, name):
            return getattr(self, name)
        else:
            raise AttributeError(name)

    def _get_loan_history(self):
        where = [tables.loans.c.movie_id == self.movie_id]
        if self.collection_id is not None:
            where.append(tables.loans.c.collection_id == self.collection_id)
        if self.volume_id is not None:
            where.append(tables.loans.c.volume_id == self.volume_id)
        return object_session(self).query(Loan).filter(
            and_(tables.loans.c.return_date is not None, or_(*where))).all()

    loan_history = property(_get_loan_history,
                            doc='list of already returned loans')

    def _get_loan_details(self):
        where = [tables.loans.c.movie_id == self.movie_id]
        if self.collection_id is not None:
            where.append(tables.loans.c.collection_id == self.collection_id)
        if self.volume_id is not None:
            where.append(tables.loans.c.volume_id == self.volume_id)
        return object_session(self).query(Loan).filter(and_(
            tables.loans.c.return_date is None, or_(*where))).first()

    loan_details = property(_get_loan_details,
                            doc='current loan details or None')

    def loan_to(self, person, whole_collection=False):
        """
        Loans movie, all other movies from the same volume and optionally
        movie's collection.

        :param person: Person instance or person_id.
        :param whole_collection: if True, will loan all movies from the same
            collection.
        """

        if self.loaned:
            log.warning('movie already loaned: %s', self.loan_details)
            return False

        session = object_session(self)

        if hasattr(person, 'person_id'):
            person = person.person_id
        elif not isinstance(person, int):
            raise ValueError(f"expecting int or Person instance, got "
                             f"{type(person)}")
        loan = Loan()
        loan.person_id = person
        loan.movie = self

        if whole_collection:
            if self.collection:
                # next line will update the status of all other movies in
                # collection or raise and OtherMovieAlreadyLoanedError
                self.collection.loaned = True
                loan.collection_id = self.collection_id
            else:
                log.debug('this movie doesn\'t have collection assigned, '
                          'whole_collection param ignored')

        if self.volume:
            # next line will update the status of all other movies in volume
            self.volume.loaned = True
            loan.volume_id = self.volume_id

        self.loaned = True
        session.add(loan)

        return True



# -*- coding: UTF-8 -*-
from xml.dom import minidom
import os
import sys
import urllib.request
import urllib.parse
import urllib.error
import string
import time
import base64
import hmac
from hashlib import sha256
from gutils import cmp
import logging
try:
    import timeoutsocket # http://www.timo-tasi.org/python/timeoutsocket.py
    timeoutsocket.setDefaultSocketTimeout(10)
except ImportError:
    pass
log = logging.getLogger("Griffith")

__all__ = []
"""Python wrapper


for Amazon web APIs

This module allows you to access Amazon's web APIs,
to do things like search Amazon and get the results programmatically.
Described here:
  http://www.amazon.com/webservices

You need a Amazon-provided license key to use these services.
Follow the link above to get one.  These functions will look in
several places (in this order) for the license key:
- the "license_key" argument of each function
- the module-level LICENSE_KEY variable (call setLicense once to set it)
- an environment variable called AMAZON_LICENSE_KEY
- a file called ".amazonkey" in the current directory
- a file called "amazonkey.txt" in the current directory
- a file called ".amazonkey" in your home directory
- a file called "amazonkey.txt" in your home directory
- a file called ".amazonkey" in the same directory as amazon.py
- a file called "amazonkey.txt" in the same directory as amazon.py

Sample usage:
>>> import amazon
>>> amazon.setLicense('...') # must get your own key!
>>> pythonBooks = amazon.searchByKeyword('Python')
>>> pythonBooks[0].ProductName
u'Learning Python (Help for Programmers)'
>>> pythonBooks[0].URL
...
>>> pythonBooks[0].OurPrice
...

Other available functions:
- browseBestSellers
- searchByASIN
- searchByUPC
- searchByAuthor
- searchByArtist
- searchByActor
- searchByDirector
- searchByManufacturer
- searchByListMania
- searchSimilar
- searchByWishlist

Other usage notes:
- Most functions can take product_line as well, see source for possible values
- All functions can take type="lite" to get less detail in results
- All functions can take page=N to get second, third, fourth page of results
- All functions can take license_key="XYZ", instead of setting it globally
- All functions can take http_proxy="http://x/y/z" which overrides your system setting
"""

__author__ = "Mark Pilgrim (f8dy@diveintomark.org)"
__version__ = "0.64.1"
__cvsversion__ = "$Revision: 1.1 $"[11:-2]
__date__ = "$Date: 2005/08/06 13:29:10 $"[7:-2]
__copyright__ = "Copyright (c) 2002 Mark Pilgrim"
__license__ = "Python"
# Powersearch and return object type fix by Joseph Reagle <geek@goatee.net>

# Locale support by Michael Josephson <mike@josephson.org>

# Modification to _contentsOf to strip trailing whitespace when loading Amazon key
# from a file submitted by Patrick Phalen.

# Support for specifying locale and associates ID as search parameters and
# internationalisation fix for the SalesRank integer conversion by
# Christian Theune <ct@gocept.com>, gocept gmbh & co. kg

# Support for BlendedSearch contributed by Alex Choo

LICENSE_KEY = ""
SECRET_KEY = ""
ASSOCIATE = "webservices-20"
HTTP_PROXY = None
LOCALE = "us"
# default API version is from 2005-10-05
APIVERSION='2009-03-31'

# don't touch the rest of these constants
class AmazonError(Exception):
    pass
class NoLicenseKey(Exception):
    pass
_amazonfile1 = ".amazonkey"
_amazonfile2 = "amazonkey.txt"
_licenseLocations = (
    (lambda key: key, 'passed to the function in license_key variable'),
    (lambda key: LICENSE_KEY, 'module-level LICENSE_KEY variable (call setLicense to set it)'),
    (lambda key: os.environ.get('AMAZON_LICENSE_KEY', None), 'an environment variable called AMAZON_LICENSE_KEY'),
    (lambda key: _contentsOf(os.getcwd(), _amazonfile1), '%s in the current directory' % _amazonfile1),
    (lambda key: _contentsOf(os.getcwd(), _amazonfile2), '%s in the current directory' % _amazonfile2),
    (lambda key: _contentsOf(os.environ.get('HOME', ''), _amazonfile1), '%s in your home directory' % _amazonfile1),
    (lambda key: _contentsOf(os.environ.get('HOME', ''), _amazonfile2), '%s in your home directory' % _amazonfile2),
    (lambda key: _contentsOf(_getScriptDir(), _amazonfile1), '%s in the amazon.py directory' % _amazonfile1),
    (lambda key: _contentsOf(_getScriptDir(), _amazonfile2), '%s in the amazon.py directory' % _amazonfile2)
    )
_supportedLocales = {
        "us" : (None, "ecs.amazonaws.com/onca/xml"),
        "uk" : ("uk", "ecs.amazonaws.co.uk/onca/xml"),
        "de" : ("de", "ecs.amazonaws.de/onca/xml"),
        "ca" : ("ca", "ecs.amazonaws.ca/onca/xml"),
        "fr" : ("fr", "ecs.amazonaws.fr/onca/xml"),
        "jp" : ("jp", "ecs.amazonaws.jp/onca/xml")
    }

## administrative functions
def version():
    print("""PyAmazon %(__version__)s
%(__copyright__)s
released %(__date__)s
""" % globals())

def setAssociate(associate):
    global ASSOCIATE
    ASSOCIATE=associate

def getAssociate(override=None):
    return override or ASSOCIATE

## utility functions

def _checkLocaleSupported(locale):
    if locale not in _supportedLocales:
        raise AmazonError("Unsupported locale. Locale must be one of: %s" %
            string.join(_supportedLocales, ", "))

def setLocale(locale):
    """set locale"""
    global LOCALE
    _checkLocaleSupported(locale)
    LOCALE = locale

def getLocale(locale=None):
    """get locale"""
    return locale or LOCALE

def setLicense(license_key, secret_key):
    """set license key"""
    global LICENSE_KEY, SECRET_KEY, HMAC
    LICENSE_KEY = license_key
    SECRET_KEY = secret_key
    HMAC = hmac.new(SECRET_KEY, digestmod=sha256)

def getLicense(license_key = None):
    """get license key

    license key can come from any number of locations;
    see module docs for search order"""
    for get, location in _licenseLocations:
        rc = get(license_key)
        if rc:
            return rc
    raise NoLicenseKey('get a license key at http://www.amazon.com/webservices')

def setProxy(http_proxy):
    """set HTTP proxy"""
    global HTTP_PROXY
    HTTP_PROXY = http_proxy

def getProxy(http_proxy = None):
    """get HTTP proxy"""
    return http_proxy or HTTP_PROXY

def getProxies(http_proxy = None):
    http_proxy = getProxy(http_proxy)
    if http_proxy:
        proxies = {"http": http_proxy}
    else:
        proxies = None
    return proxies

def _contentsOf(dirname, filename):
    filename = os.path.join(dirname, filename)
    if not os.path.exists(filename):
        return None
    fsock = open(filename)
    contents =  fsock.read().strip()
    fsock.close()
    return contents

def _getScriptDir():
    if __name__ == '__main__':
        return os.path.abspath(os.path.dirname(sys.argv[0]))
    else:
        return os.path.abspath(os.path.dirname(sys.modules[__name__].__file__))

class Bag:
    pass

def unmarshal(element):
    rc = Bag()
    if isinstance(element, minidom.Element) and (element.tagName == 'Details'):
        rc.URL = element.attributes["url"].value
    childElements = [e for e in element.childNodes if isinstance(e, minidom.Element)]
    if childElements:
        for child in childElements:
            key = child.tagName
            if hasattr(rc, key):
                if type(getattr(rc, key)) != type([]):
                    setattr(rc, key, [getattr(rc, key)])
                setattr(rc, key, getattr(rc, key) + [unmarshal(child)])
            elif isinstance(child, minidom.Element) and (child.tagName == 'Details' or child.tagName == 'Item'):
                # make the first Details element a key
                setattr(rc,key,[unmarshal(child)])
                #dbg: because otherwise 'hasattr' only tests
                #dbg: on the second occurence: if there's a
                #dbg: single return to a query, it's not a
                #dbg: list. This module should always
                #dbg: return a list of Details objects.
            else:
                setattr(rc, key, unmarshal(child))
    else:
        rc = "".join([e.data for e in element.childNodes if isinstance(e, minidom.Text)])
        if element.tagName == 'SalesRank':
            rc = rc.replace('.', '')
            rc = rc.replace(',', '')
            rc = int(rc)
    return rc

def buildURL(search_type, searchfield, searchvalue, product_line, type, page, license_key, locale, associate, timestamp = None):
    _checkLocaleSupported(locale)
    if isinstance(searchvalue, str):
        searchvalue = searchvalue.encode('utf-8') # needed for urllib.quote
    url = "http://" + _supportedLocales[locale][1]
    if search_type == 'ItemLookup':
        params = dict(
            Service='AWSECommerceService',
            AWSAccessKeyId=license_key,
            AssociateTag=associate,
            Operation=search_type,
            ResponseGroup=type,
            IdType=searchfield,
            ItemId=urllib.parse.quote(searchvalue))
        if product_line:
            params['SearchIndex'] = product_line
    else:
        params = dict(
            Service='AWSECommerceService',
            AWSAccessKeyId=license_key,
            AssociateTag=associate,
            Operation=search_type,
            ResponseGroup=type,
            Sort='titlerank')
        params[searchfield] = urllib.parse.quote(searchvalue)
        if page:
            params['ItemPage'] = str(page)
        if product_line:
            params['SearchIndex'] = product_line
    if APIVERSION:
        params['Version'] = APIVERSION
    log.info(params)
    url = getSignedUrl(url, params, timestamp)
    log.info('URL: ' + url)
    return url

def getSignedUrl(base_url, params, timestamp = None):
    global HMAC
    hmac = HMAC.copy()

    # Add a ISO 8601 compliant timestamp (in GMT)
    if timestamp:
        params['Timestamp'] = timestamp
    else:
        params['Timestamp'] = time.strftime("%Y-%m-%dT%H:%M:%S", time.gmtime())
    #params['SignatureVersion']='1'

    # Sort the URL parameters by key
    keys = list(params.keys())
    #keys.sort(cmp = lambda x, y: cmp(x.lower(), y.lower()))
    keys.sort()

    # Reconstruct the URL parameters and encode them
    pairs = []
    for key in keys:
        #val = urllib.quote(params[key])
        val = params[key]
        pairs.append(key + '=' + val)
    url_string = '&'.join(pairs)
    url_string = url_string.replace('+', "%20")
    url_string = url_string.replace(':', "%3A")

    #Construct the string to sign
    urlparts = base_url.split('/')
    string_to_sign = """GET
%s
/%s/%s
%s""" % (urlparts[2], urlparts[3], urlparts[4], url_string)
    url_string = url_string.replace(';', urllib.parse.quote(';'))

    # Sign the request
    hmac.update(string_to_sign)
    signature = hmac.digest()

    # Base64 encode the signature
    signature = base64.encodestring(signature).strip()
    signature = signature.replace('+','%2B')
    signature = signature.replace('=','%3D')
    signature = signature.replace('/','%2F')

    # Make the signature URL safe
    url_string += "&Signature=%s" % signature
    return "%s?%s" % (base_url, url_string)


## main functions


def search(search_type, searchfield, searchvalue, product_line, type = "Large", page = None,
           license_key=None, http_proxy = None, locale = None, associate = None):
    """search Amazon

    You need a license key to call this function; see
    http://www.amazon.com/webservices
    to get one.  Then you can either pass it to
    this function every time, or set it globally; see the module docs for details.

    Parameters:
    keyword - keyword to search
    search_type - in (KeywordSearch, BrowseNodeSearch, AsinSearch, UpcSearch, AuthorSearch, ArtistSearch, ActorSearch, DirectorSearch, ManufacturerSearch, ListManiaSearch, SimilaritySearch)
    product_line - type of product to search for.  restrictions based on search_type
        UpcSearch - in (music, classical)
        AuthorSearch - must be "books"
        ArtistSearch - in (music, classical)
        ActorSearch - in (dvd, vhs, video)
        DirectorSearch - in (dvd, vhs, video)
        ManufacturerSearch - in (electronics, kitchen, videogames, software, photo, pc-hardware)
    http_proxy (optional) - address of HTTP proxy to use for sending and receiving SOAP messages

    Returns: list of Bags, each Bag may contain the following attributes:
      Asin - Amazon ID ("ASIN" number) of this item
      Authors - list of authors
      Availability - "available", etc.
      BrowseList - list of related categories
      Catalog - catalog type ("Book", etc)
      CollectiblePrice - ?, format "$34.95"
      ImageUrlLarge - URL of large image of this item
      ImageUrlMedium - URL of medium image of this item
      ImageUrlSmall - URL of small image of this item
      Isbn - ISBN number
      ListPrice - list price, format "$34.95"
      Lists - list of ListMania lists that include this item
      Manufacturer - manufacturer
      Media - media ("Paperback", "Audio CD", etc)
      NumMedia - number of different media types in which this item is available
      OurPrice - Amazon price, format "$24.47"
      ProductName - name of this item
      ReleaseDate - release date, format "09 April, 1999"
      Reviews - reviews (AvgCustomerRating, plus list of CustomerReview with Rating, Summary, Content)
      SalesRank - sales rank (integer)
      SimilarProducts - list of Product, which is ASIN number
      ThirdPartyNewPrice - ?, format "$34.95"
      URL - URL of this item
    """
    license_key = getLicense(license_key)
    locale = getLocale(locale)
    associate = getAssociate(associate)
    url = buildURL(search_type, searchfield, searchvalue, product_line, type, page,
            license_key, locale, associate)
    proxies = getProxies(http_proxy)
    u = urllib.request.FancyURLopener(proxies)
    usock = u.open(url)
    xmldoc = minidom.parse(usock)

    usock.close()
    data = unmarshal(xmldoc)
    if search_type == "BlendedSearch":
        if hasattr(data, 'BlendedSearch'):
            data = data.BlendedSearch
    elif hasattr(data, 'ItemSearchResponse'):
        data = data.ItemSearchResponse
    elif hasattr(data, 'ItemLookupResponse'):
        data = data.ItemLookupResponse

    if hasattr(data, 'Errors'):
        raise AmazonError(data.Errors)
    else:
        if search_type == "BlendedSearch":
            return data
        else:
            if hasattr(data, 'Items'):
                return data.Items
            else:
                return data

def searchByKeyword(keyword, product_line="Books", type="Large", page=1, license_key=None, http_proxy=None, locale=None, associate=None):
    return search("ItemSearch", 'Keywords', keyword, product_line, type, page, license_key, http_proxy, locale, associate)

def searchByTitle(keyword, product_line="Books", type="Large", page=1, license_key=None, http_proxy=None, locale=None, associate=None):
    return search("ItemSearch", 'Title', keyword, product_line, type, page, license_key, http_proxy, locale, associate)

def browseBestSellers(browse_node, product_line="Books", type="Large", page=1, license_key=None, http_proxy=None, locale=None, associate=None):
    return search("BrowseNodeSearch", 'Keywords', browse_node, product_line, type, page, license_key, http_proxy, locale, associate)

def searchByASIN(ASIN, type="Large", license_key=None, http_proxy=None, locale=None, associate=None):
    return search("ItemLookup", 'ASIN', ASIN, None, type, None, license_key, http_proxy, locale, associate)

def searchByUPC(UPC, product_line="Books", type="Large", license_key=None, http_proxy=None, locale=None, associate=None):
    return search("ItemLookup", 'UPC', UPC, product_line, type, None, license_key, http_proxy, locale, associate)

def searchByEAN(EAN, product_line="Books", type="Large", license_key=None, http_proxy=None, locale=None, associate=None):
    return search("ItemLookup", 'EAN', EAN, product_line, type, None, license_key, http_proxy, locale, associate)

def searchByAuthor(author, type="Large", page=1, license_key=None, http_proxy=None, locale=None, associate=None):
    return search("AuthorSearch", 'Keywords', author, "Books", type, page, license_key, http_proxy, locale, associate)

def searchByArtist(artist, product_line="Music", type="Large", page=1, license_key=None, http_proxy=None, locale=None, associate=None):
    if product_line not in ("music", "classical"):
        raise AmazonError("product_line must be in ('Music', 'Classical')")
    return search("ArtistSearch", 'Keywords', artist, product_line, type, page, license_key, http_proxy, locale, associate)

def searchByActor(actor, product_line="DVD", type="Large", page=1, license_key=None, http_proxy=None, locale=None, associate=None):
    if product_line not in ("DVD", "VHS", "Video"):
        raise AmazonError("product_line must be in ('DVD', 'VHS', 'Video')")
    return search("ActorSearch", 'Keywords', actor, product_line, type, page, license_key, http_proxy, locale, associate)

def searchByDirector(director, product_line="DVD", type="Large", page=1, license_key=None, http_proxy=None, locale=None, associate=None):
    if product_line not in ("DVD", "VHS", "Video"):
        raise AmazonError("product_line must be in ('DVD', 'VHS', 'Video')")
    return search("DirectorSearch", 'Keywords', director, product_line, type, page, license_key, http_proxy, locale, associate)

def searchByManufacturer(manufacturer, product_line="pc-hardware", type="Large", page=1, license_key=None, http_proxy=None, locale=None, associate=None):
    if product_line not in ("electronics", "kitchen", "videogames", "software", "photo", "pc-hardware"):
        raise AmazonError("product_line must be in ('electronics', 'kitchen', 'videogames', 'software', 'photo', 'pc-hardware')")
    return search("ManufacturerSearch", 'Keywords', manufacturer, product_line, type, page, license_key, http_proxy, locale, associate)

def searchByListMania(listManiaID, type="Large", page=1, license_key=None, http_proxy=None, locale=None, associate=None):
    return search("ListManiaSearch", 'Keywords', listManiaID, None, type, page, license_key, http_proxy, locale, associate)

def searchSimilar(ASIN, type="Large", page=1, license_key=None, http_proxy=None, locale=None, associate=None):
    return search("SimilaritySearch", 'Keywords', ASIN, None, type, page, license_key, http_proxy, locale, associate)

def searchByWishlist(wishlistID, type="Large", page=1, license_key=None, http_proxy=None, locale=None, associate=None):
    return search("WishlistSearch", 'Keywords', wishlistID, None, type, page, license_key, http_proxy, locale, associate)

def searchByPower(keyword, product_line="Books", type="Large", page=1, license_key=None, http_proxy=None, locale=None, associate=None):
    return search("PowerSearch", 'Keywords', keyword, product_line, type, page, license_key, http_proxy, locale, associate)
    # >>> RecentKing = amazon.searchByPower('author:Stephen King and pubdate:2003')
    # >>> SnowCrash = amazon.searchByPower('title:Snow Crash')

def searchByBlended(keyword, type="Large", page=1, license_key=None, http_proxy=None, locale=None, associate=None):
    return search("BlendedSearch", 'Keywords', keyword, None, type, page, license_key, http_proxy, locale, associate)

if __name__ == "__main__":
    # check the generation of a sample url
    expectedUrl = 'http://ecs.amazonaws.de/onca/xml?AWSAccessKeyId=ABCDEFGHIJKLMNOPQRST&AssociateTag=webservices-20&IdType=Title&ItemId=Prison%C2%A0Break&ItemPage=1&Operation=ItemSearch&ResponseGroup=Large&SearchIndex=DVD&Service=AWSECommerceService&Sort=titlerank&Timestamp=2009-11-04T13%3A12%3A51&Title=Prison%C2%A0Break&Version=2008-06-26&Signature=zCHdyoI5busTseJrYcVO%2Bz6u9pmULHBp3E%2F%2FGAdAM2g%3D'
    setLicense('ABCDEFGHIJKLMNOPQRST', 'ancnhH/kdfsk739/dfsjkjikJHFJKJHRih7hHZgd')
    url = buildURL('ItemSearch', 'Title', 'Prison Break', 'DVD', 'Large', 1, 'ABCDEFGHIJKLMNOPQRST', 'de', 'webservices-20', '2009-11-04T13%3A12%3A51')
    # compare with a generated one from http://associates-amazon.s3.amazonaws.com/signed-requests/helper/index.html
    result = cmp(url, expectedUrl)
    if result == 0:
        print('Signed URL generation seems to be ok.')
    else:
        print(url)
        print(expectedUrl)
        print('Something seems to be wrong with the signed URL generation.')



# -*- coding: UTF-8 -*-

__revision__ = '$Id$'

# Copyright (c) 2005-2007 Vasco Nunes

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

import smtplib
import socket
from gettext import gettext as _

import gutils

socket.setdefaulttimeout(10)


def mailto(self, tls, port, server, auth, user, password, sender, to, subject, msg):
    """
    sends an email
    """
    try:
        session = smtplib.SMTP(server, port)
        session.set_debuglevel(1)
    except socket.timeout:
        gutils.error(_("Connection timed out"), \
            self.widgets['window'])
        return()
    if tls is True:
        session.ehlo()
        session.starttls()
        session.ehlo()
    if auth:
        try:
            session.login(user, password)
        except Exception:
            gutils.error(_("Error sending e-mail: %s")%_("login failure"), \
                self.widgets['window'])
            return
    headers = "From: %s\r\nTo: %s\r\nSubject: %s\r\n\r\n" \
        % (sender, to, subject)
    try:
        session.sendmail(sender, to, headers+msg)
        gutils.info(_("E-mail sent successfully"), self.widgets['window'])
        return
    except Exception:
        gutils.error(_("Error sending e-mail: %s")%"", self.widgets['window'])

    session.quit()

def send_email(self):
    if len(self.person_email):
        if self.config.get('use_auth', False, section='mail') is True:
            use_auth = 1
        else:
            use_auth = 0
        try:
            mailto(self, self.config.get('mail_use_tls', False, section='mail'), \
                self.config.get('mail_smtp_port', '25', section='mail'), \
                self.config.get('smtp_server', 'localhost', section='mail'), \
                use_auth, self.config.get('username', '', section='mail'), \
                self.config.get('password', '', section='mail'), \
                self.config.get('email', 'griffith', section='mail'), self.person_email, \
                _("Movie loan reminder"), _("Hi, %s!\n\nJust to remind you that I'm really needing the following movie I have loaned to you recently:\n\n%s (%s)\n\nLoaned on %s") \
                %(self.person_name, self.widgets['movie']['o_title'].get_text(), \
                self.widgets['movie']['title'].get_text(), self.loan_date[:10]))
        except Exception:
            gutils.error(_("Mail could not be sent. Please check e-mail preferences."), self.widgets['window'])
    else:
        gutils.info(_("This person has no e-mail address defined."), \
            self.widgets['window'])



# -*- coding: UTF-8 -*-

__revision__ = '$Id$'

# Copyright (c) 2005-2009 Vasco Nunes, Piotr Ożarowski

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

import gutils
import db

def change_filter(self):
    from sqlalchemy import select
    statement = select(db.tables.movies)
    
    change_filter_update_whereclause(self, statement)
    self.populate_treeview(statement)


def change_filter_update_whereclause(self, statement):
    from sqlalchemy import or_
    text = gutils.gescape(self.widgets['filter']['text'].get_text())
    if text:
        (criterianame, criteria) = self.search_criteria_sorted[self.widgets['filter']['criteria'].get_active()]
        if criteria in ('year', 'runtime', 'media_num', 'rating'):
            statement = statement.where(db.tables.movies.c[criteria] == text)
        elif criteria == 'any':
            crits = [ ]
            for crit in ( 'director', 'title', 'o_title', 'cameraman', 'cast', 'year', 'screenplay', 'genre', 'studio', 'classification', 'country' ):
                crits.append(db.tables.movies.c[crit].like('%'+text+'%'))
            statement = statement.where(or_(*crits))
        else:
            statement = statement.where(db.tables.movies.c[criteria].like('%' + text + '%'))
    if self.widgets['filter']['text'].is_focus():
        if len(text)<4: # filter mode
            limit = int(self.config.get('limit', 0, section='mainlist'))
            if limit > 0:
                statement = statement.limit(limit)

    
def clear_filter(self, populate=True):
    # prevent multiple treeview updates
    self.initialized = False
    self.widgets['filter']['text'].set_text('')
    self.widgets['filter']['collection'].set_active(0)
    self.widgets['filter']['advfilter'].set_active(0)
    self.initialized = True
    if populate:
        self.populate_treeview()



# -*- coding: UTF-8 -*-

__revision__ = '$Id$'

# Copyright © 2005-2010 Vasco Nunes, Piotr Ożarowski
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later
from gettext import gettext as _

pname = 'Griffith'
pversion = '0.24'
pauthor = 'Vasco Nunes, Piotr Ożarowski <griffith@griffith.cc>, Dan Johansen <strit@strits.dk>'
pyear = '2005-2024'
pwebsite = 'https://gitlab.com/Strit/griffith/'
pdescription = _('Griffith is a film collection manager.')



# -*- coding: UTF-8 -*-

__revision__ = '$Id$'

# Copyright (c) 2006-2012
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

import gutils
import movie
import re
from gettext import gettext as _

plugin_name         = "DVD-Palace"
plugin_description  = "DVD-Onlinemagazin mit DVD-Datenbank"
plugin_url          = "www.dvd-palace.de"
plugin_language     = _("German")
plugin_author       = "Michael Jahn"
plugin_author_email = "<mikej06@hotmail.com>"
plugin_version      = "1.3"

class Plugin(movie.Movie):

    def __init__(self, id):
        self.encode   = 'iso-8859-1'
        self.movie_id = id
        self.url      = 'http://www.dvd-palace.de/datenbank/medien/' + self.movie_id

    def get_image(self):
        self.image_url = gutils.trim(self.page, 'src="/showcover.php?', '"')
        if self.image_url != '':
            self.image_url = 'http://www.dvd-palace.de/showcover.php?' + self.image_url

    def get_o_title(self):
        self.o_title = gutils.trim(self.page, 'Originaltitel', '</b>')
        if self.o_title == '':
            self.o_title = gutils.trim(self.page, '<TITLE>', ' - Details')

    def get_title(self):
        self.title = gutils.trim(self.page, '<TITLE>', ' - Details')

    def get_director(self):
        self.director = gutils.trim(self.page, 'Regisseur(e)', '</TR>')

    def get_plot(self):
        self.plot = re.sub(
                '[]', '-',
                re.sub(
                    '[\x93]', '"', gutils.regextrim(self.page, '[0-9 ]+Views', '</td>')))

    def get_year(self):
        self.year = gutils.after(gutils.trim(gutils.trim(self.page, 'Originaltitel', '</TR>'), '(', ')'), ' ')

    def get_runtime(self):
        self.runtime = gutils.strip_tags(gutils.trim(self.page, 'Laufzeit', ' min').replace('ca. ', ''))

    def get_genre(self):
        tempstr = gutils.trim(self.page, 'Genre(s)', '</TR>').replace('&nbsp;', '')
        self.genre = tempstr.replace('<br>', ', ')

    def get_cast(self):
        tempstr = re.sub('<em>[^<]*</em>', ' ',
                     re.sub(',[ ]*', '\n',
                         gutils.trim(self.page, 'Darsteller / Sprecher', '</TR>')
                )
            )
        self.cast = tempstr.replace('\n ', '\n')

    def get_classification(self):
        self.classification = gutils.trim(self.page, 'Altersfreigabe (FSK)', '</TR>').replace('&nbsp;', '')

    def get_studio(self):
        tempstr = gutils.trim(self.page, 'Label</b>', '</td>').replace('&nbsp;', '')
        self.studio = tempstr.replace(':', '')
        if not self.studio:
            tempstr = gutils.trim(self.page, 'Vertrieb</b>', '</td>').replace('&nbsp;', '')
            self.studio = tempstr.replace(':', '')

    def get_o_site(self):
        self.o_site = ""

    def get_site(self):
        self.site = self.url

    def get_trailer(self):
        self.trailer = ''

    def get_country(self):
        self.country = gutils.before(gutils.trim(gutils.trim(self.page, 'Originaltitel', '</TR>'), '(', ')'), ' ')

    def get_rating(self):
        self.rating = "0"

    def get_notes(self):
        self.notes = ""
        tmp_notes = re.sub('^[ \t]+', '',
            gutils.strip_tags(
            re.sub('(<br>|<br />)', '\r\n',
            re.sub('[\r\n]+', '',
            re.sub('[ \t][ \t\r\n]+', ' ',
            gutils.trim(self.page, 'Bildformat(e)', '</TR>')))))
        )
        if (tmp_notes != ""):
            self.notes = self.notes + "Bildformat(e):\n" + tmp_notes + "\n"
        tmp_notes = re.sub('^[ \t]+', '',
            gutils.strip_tags(
            re.sub('(<br>|<br />)', '\r\n',
            re.sub('[\r\n]+', '',
            re.sub('[ \t][ \t\r\n]+', ' ',
            gutils.trim(self.page, 'Tonformat(e)', '</TR>')))))
        )
        if (tmp_notes != ""):
            self.notes = self.notes + "Tonformat(e):\n" + tmp_notes + "\n\n"
        tmp_notes = re.sub('^[ \t]+', '',
            gutils.strip_tags(
            re.sub('(<br>|<br />)', '\r\n',
            re.sub('[\r\n]+', '',
            re.sub('[ \t][ \t\r\n]+', ' ',
            gutils.trim(self.page, 'Untertitel', '</TR>')))))
        )
        if (tmp_notes != ""):
            self.notes = self.notes + "Untertitel:" + tmp_notes + "\n\n"

    def get_barcode(self):
        self.barcode = gutils.trim(self.page, 'EAN', '</b>')


class SearchPlugin(movie.SearchMovie):

    translations = (
        ('ü', 'ue'),
        ('ä', 'ae'),
        ('ö', 'oe'),
        ('Ü', 'UE'),
        ('Ä', 'AE'),
        ('Ö', 'OE'),
        ('ß', 'ss')
    )

    def __init__(self):
        self.original_url_search   = "http://www.dvd-palace.de/dvddatabase/dbsearch.php?action=1&suchbegriff="
        self.translated_url_search = "http://www.dvd-palace.de/dvddatabase/dbsearch.php?action=1&suchbegriff="
        self.encode='iso-8859-1'
        self.remove_accents = False

    def search(self,parent_window):
        # replace some standard german umlauts
        for from_str, to_str in self.translations:
            self.title = self.title.replace(from_str, to_str)
        # AFTER that remove accents
        self.title = gutils.remove_accents(str(self.title))
        if not self.open_search(parent_window):
            return None
        tmp_pagemovie = self.page
        #
        # try to get all result pages (not so nice, but it works)
        #
        tmp_pagecount = gutils.trim(tmp_pagemovie, 'Seiten (<b>', '</b>)')
        try:
            tmp_pagecountint = int(tmp_pagecount)
        except Exception:
            tmp_pagecountint = 1
        tmp_pagecountintcurrent = 1
        while (tmp_pagecountint > tmp_pagecountintcurrent and tmp_pagecountintcurrent < 5):
            self.url = "http://www.dvd-palace.de/dvddatabase/dbsearch.php?action=1&start=" + str(tmp_pagecountintcurrent * 20) + "&suchbegriff="
            if self.open_search(parent_window):
                tmp_pagemovie = tmp_pagemovie + self.page
                tmp_pagecountintcurrent = tmp_pagecountintcurrent + 1
        self.page = tmp_pagemovie
        return self.page

    def get_searches(self):
        elements = re.split('&nbsp;<a title="[^"]+" href="(/datenbank/medien/dvd/|/datenbank/medien/blu-ray/)', self.page)
        elements[0] = None
        for index in range(1, len(elements), 2):
            element = elements[index + 1]
            if element is not None:
                if elements[index] == '/datenbank/medien/blu-ray/':
                    medium = 'Blu-Ray'
                    self.ids.append('blu-ray/' + gutils.before(element,'"'))
                else:
                    medium = 'DVD'
                    self.ids.append('dvd/' + gutils.before(element,'"'))
                tempstr = gutils.regextrim(element, '<div [^>]*>', '</div>').replace('<br>', ' - ')
                self.titles.append(
                    gutils.trim(element, '>', '</a>') +
                    gutils.clean(
                        '(' + medium + ' - ' +
                        re.sub('[ \t\n]+', ' ',
                        tempstr.replace('&nbsp;', ''))
                        + ')'
                    )
                )



# -*- coding: UTF-8 -*-

__revision__ = '$Id$'

# Copyright (c) 2006-2011
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

import gi
import gettext
import sys
import initialize
import gutils
import gconsole
import config
import os
from time import sleep
import logging
from pathlib import Path
from gi.repository import Gtk
gi.require_version('Gtk', '3.0')

"""
The code within this file is only used to automatically test movie plugins
which support that.
The movie plugin which should be tested has to be added to the
PluginTester.test_plugins list and has to define to classes
SearchPluginTest and PluginTest.
Both classes provide a member called test_configuration which is a
dict in both cases.

SearchPluginTest.test_configuration:
dict { movie_id -> [ expected result count for original url, expected result count for translated url ] }

PluginTest.test_configuration:
dict { movie_id -> dict { attribute -> value } }

value: * True/False if attribute should only be tested for any value or nothing
       * or the expected value
"""

gettext.install('griffith')

logging.basicConfig()
log = logging.getLogger('Griffith')

sys.path.append('plugins/movie')
sys.path.append('plugins/extensions')


def get_plugin_list():
    curr_dir = Path(__file__).absolute().parent
    plugin_dir = curr_dir / 'plugins' / 'movie'
    plugin_list = []
    for plugin in plugin_dir.glob("Plugin*.py"):
        plugin_list.append(plugin.stem)
    return sorted(plugin_list)


def dialog_test_plugin(plugin: str = None):
    """Ask if either all or one plugin should be tested"""
    if plugin:
        msg = f"Test plugin {plugin}?"
    else:
        msg = 'Test all plugins?'
    dialog = Gtk.MessageDialog(
        modal=True, destroy_with_parent=True,
        message_type=Gtk.MessageType.QUESTION, buttons=Gtk.ButtonsType.NONE,
        text=msg
        )
    dialog.add_buttons(Gtk.STOCK_YES, Gtk.ResponseType.YES,
                       Gtk.STOCK_NO, Gtk.ResponseType.NO)
    dialog.set_default_response(Gtk.ResponseType.NO)
    dialog.set_skip_taskbar_hint(False)
    response = dialog.run()
    dialog.destroy()
    return response


class PluginTester:
    """
    test class for movie plugin classes Plugin and SearchPlugin
    it simulates the resolving of movie data for configured movies and
    compares the results with the expected one
    """

    # test_plugins = [
    #     'PluginMovieAllocine',
    #     'PluginMovieAllRovi',
    #     'PluginMovieAmazon',
    #     'PluginMovieAniDB',
    #     'PluginMovieCinematografo',
    #     'PluginMovieCineMovies',
    #     'PluginMovieCineteka',
    #     'PluginMovieClubedevideo',
    #     'PluginMovieCSFD',
    #     'PluginMovieCulturalia',
    #     'PluginMovieDVDEmpire',
    #     'PluginMovieE-Pipoca',
    #     'PluginMovieFilmAffinity',
    #     'PluginMovieFilmeVonAZ',
    #     'PluginMovieFilmtipset',
    #     'PluginMovieHKMDB',
    #     'PluginMovieIMDB',
    #     'PluginMovieIMDB-de',
    #     'PluginMovieIMDB-es',
    #     'PluginMovieKinoDe',
    #     'PluginMovieMovieMeter',
    #     'PluginMovieMyMoviesIt',
    #     'PluginMovieOFDb',
    #     'PluginMovieScope',
    #     'PluginMovieZelluloid',
    #     ]
    test_plugins = get_plugin_list()

    def __init__(self):
        self._tmp_home = None
        self.movie = None
        self.locations = None

    def test_search(self, plugin, logfile, title, cnt_original, cnt_translated):
        """
        Simulates the search for a movie title and compares  the count of
        results with the expected count
        """
        global myconfig
        result = True
        plugin.config = myconfig
        plugin.locations = self.locations
        # plugin.translated_url_search
        plugin.url = plugin.translated_url_search
        if plugin.remove_accents:
            plugin.title = gutils.remove_accents(title, 'utf-8')
        else:
            plugin.title = title
        plugin.search_movies(None)
        plugin.get_searches()
        if not len(plugin.ids) - 1 == cnt_original:    # first entry is always '' (???)
            msg = (f"Title (Translated): {title} - expected: {cnt_original} - "
                   f"found: {len(plugin.ids) - 1}")
            print(msg)
            logfile.write(f"{msg}\n\n")
            for titleFound in plugin.titles:
                logfile.write(titleFound)
                logfile.write('\n')
            logfile.write('\n\n')
            result = False
        # plugin.original_url_search if it is different to
        # plugin.translated_url_search
        if plugin.original_url_search != plugin.translated_url_search:
            plugin.url = plugin.original_url_search
            if plugin.remove_accents:
                plugin.title = gutils.remove_accents(title, 'utf-8')
            else:
                plugin.title = title
            plugin.search_movies(None)
            plugin.get_searches()
            if not len(plugin.ids) - 1 == cnt_translated:    # first entry is always '' (???)
                msg = (f"Title (Original): {title} - expected: {cnt_translated} "
                       f"- found: {len(plugin.ids) - 1}")
                print(msg)
                logfile.write(f"{msg}\n\n")
                for titleFound in plugin.titles:
                    logfile.write(f"{titleFound}\n")
                logfile.write('\n\n')
                result = False
        return result

    #
    # check every configured movie title
    #
    def do_test_search_plugin(self, plugin_name, do_msg_box=True):
        result = True

        try:
            plugin = __import__(plugin_name, globals())
        except ModuleNotFoundError as exc:
            log.info(f"Plugin '{plugin_name}' not found: {exc}")
            return False
        try:
            plugin_test_config = plugin.SearchPluginTest()
        except Exception as exc:
            msg = (f"Warning: SearchPlugin test could not be executed for "
                   f"{plugin_name} because of missing configuration class "
                   f"SearchPluginTest. {type(exc).__name__}")
            print(msg)
            return False

        if plugin_test_config is not None:
            with open(f"{plugin_name}-searchtest.log", 'w+') as log_file:
                for i in plugin_test_config.test_configuration:
                    search_plugin = plugin.SearchPlugin()
                    if not self.test_search(
                            search_plugin, log_file, i,
                            plugin_test_config.test_configuration[i][0],
                            plugin_test_config.test_configuration[i][1]):
                        result = False
                    sleep(1)  # needed for amazon

        if do_msg_box:
            if not result:
                gutils.error('SearchPluginTest %s: Test NOT successful !' %
                             plugin_name)
            else:
                gutils.info('SearchPluginTest %s: Test successful !' %
                            plugin_name)

        return result

    #
    # simulates the resolving of movie data for configured movies and
    # compares the results with the expected once
    #
    def test_one_movie(self, movie_plugin, log_file, results_expected):
        global myconfig
        result = True
        self.movie = movie_plugin
        self.movie.parent_window = None
        self.movie.locations = self.locations
        self.movie.config = myconfig

        fields_to_fetch = [
            'o_title', 'title', 'director', 'plot', 'cast', 'country', 'genre',
            'classification', 'studio', 'o_site', 'site', 'trailer', 'year',
            'notes', 'runtime', 'image', 'rating', 'screenplay', 'cameraman',
            'resolution', 'barcode']

        self.movie.fields_to_fetch = fields_to_fetch

        self.movie.get_movie(None)
        self.movie.parse_movie()

        results = {}
        if 'year' in fields_to_fetch:
            results['year'] = self.movie.year
            fields_to_fetch.pop(fields_to_fetch.index('year'))
        if 'runtime' in fields_to_fetch:
            results['runtime'] = self.movie.runtime
            fields_to_fetch.pop(fields_to_fetch.index('runtime'))
        if 'cast' in fields_to_fetch:
            results['cast'] = gutils.convert_entities(self.movie.cast)
            fields_to_fetch.pop(fields_to_fetch.index('cast'))
        if 'plot' in fields_to_fetch:
            results['plot'] = gutils.convert_entities(self.movie.plot)
            fields_to_fetch.pop(fields_to_fetch.index('plot'))
        if 'notes' in fields_to_fetch:
            results['notes'] = gutils.convert_entities(self.movie.notes)
            fields_to_fetch.pop(fields_to_fetch.index('notes'))
        if 'rating' in fields_to_fetch:
            if self.movie.rating:
                results['rating'] = float(self.movie.rating)
            fields_to_fetch.pop(fields_to_fetch.index('rating'))
        # poster
        if 'image' in fields_to_fetch:
            if self.movie.image:
                results['image'] = self.movie.image
            fields_to_fetch.pop(fields_to_fetch.index('image'))
        # other fields
        for i in fields_to_fetch:
            results[i] = gutils.convert_entities(self.movie[i])

        # check the fields
        for i in results_expected:
            i_val = None
            try:
                i_val = results_expected[i]
                if isinstance(i_val, bool):
                    if i_val:
                        # if not result or i not in results or len(results[i]) < 1:
                        if i not in results or len(results[i]) < 1:
                            msg = (f"Test error: {movie_plugin.movie_id}: Value "
                                   f"expected but nothing returned.\nKey: {i}")
                            print(msg)
                            log_file.write(f"{msg}\n\n")
                            result = False
                    else:
                        if i in results:
                            if isinstance(results[i], int) and results[i] == 0:
                                continue
                            if (not isinstance(results[i], int)
                                    and len(results[i]) < 1):
                                continue
                            msg = (f"Test error: {movie_plugin.movie_id}: No "
                                   f"value expected but something returned.\n"
                                   f"Key: {i}\nValue: {results[i]}")
                            print(msg)
                            log_file.write(f"{msg}\n\n")
                            result = False
                else:
                    if i not in results:
                        msg = (f"Test error: {movie_plugin.movie_id}: Value "
                               f"expected but nothing returned.\nKey: {i}")
                        print(msg)
                        log_file.write(f"{msg}\n\n")
                        result = False
                    else:
                        if not results[i] == i_val:
                            msg = (f"Test error: {movie_plugin.movie_id}: Wrong "
                                   f"value returned.\nKey: {i}\nValue expected:"
                                   f" {i_val}\nValue returned: {results}")
                            print(msg)
                            log_file.write(f"{msg}\n\n")
                            result = False
            except Exception:
                log.exception(f"{i} - {i_val} - {results[i]} - {movie_plugin}")
                result = False
        return result

    def do_test_plugin(self, plugin_name, do_msg_box=True):
        """
        check every configured movie
        """
        result = True

        plugin = __import__(plugin_name)
        try:
            plugin_test_config = plugin.PluginTest()
        except Exception:
            print(f"Warning: Plugin test could not be executed for "
                  f"{plugin_name} because of missing configuration class "
                  "PluginTest.")
            plugin_test_config = None

        if plugin_test_config is not None:
            log_file = open(f"{plugin_name}-loadtest.log", 'w+')
            try:
                for i in plugin_test_config.test_configuration:
                    movie_plugin = plugin.Plugin(i)
                    if not self.test_one_movie(
                            movie_plugin, log_file,
                            plugin_test_config.test_configuration[i]):
                        result = False
                    sleep(1)  # needed for amazon
            finally:
                log_file.close()

        if do_msg_box:
            if not result:
                gutils.error(f"PluginTest {plugin_name}: Test NOT successful !")
            else:
                gutils.info(f"PluginTest {plugin_name}: Test successful !")

        return result

    def do_tests(self, do_msg_box=True):
        """
        main method, iterates through all plugins which should be auto-tested
        and executes the Plugin and SearchPlugin test methods
        """
        global myconfig
        self._tmp_home = None
        home_dir, config_name = gconsole.check_args()
        if not (config_name.find('/') >= 0 or config_name.find('\\') >= 0):
            config_name = os.path.join(home_dir, config_name)
        log.debug("config file used: %s", config_name)
        myconfig = config.Config(file=config_name)
        initialize.locations(self, home_dir)

        gettext.install('griffith', self.locations['i18n'])

        search_successful, search_unsuccessful = [], []
        get_successful, get_unsuccessful = [], []

        test_all = True  # test all plugins ?
        response = dialog_test_plugin()
        if not response == Gtk.ResponseType.YES:
            test_all = False
        # iterate through all testable plugins
        for plugin in self.test_plugins:
            if test_all is False and do_msg_box:
                response = dialog_test_plugin(plugin)
                if not response == Gtk.ResponseType.YES:
                    continue
            print(f"\n___ Starting test of plugin: {plugin}")
            try:
                _plugin = __import__(plugin)
            except ModuleNotFoundError as exc:
                msg = f"Plugin '{plugin}' not found: {exc}"
                log.error(msg)
                print(msg)
                continue
            # search test
            if self.do_test_search_plugin(plugin, False):
                search_successful.append(plugin)
            else:
                search_unsuccessful.append(plugin)
            # movie test
            if self.do_test_plugin(plugin, False):
                get_successful.append(plugin)
            else:
                get_unsuccessful.append(plugin)
        if do_msg_box:
            gutils.info(
                f"SearchPluginTests\n Success: {', '.join(search_successful)}\n"
                f" Failed: {', '.join(search_unsuccessful)}\n\n"
                f"PluginTests\n Success: {', '.join(get_successful)}\n"
                f" Failed: {', '.join(get_unsuccessful)}")


if __name__ == "__main__":
    PluginTester().do_tests()



# -*- coding: UTF-8 -*-

__revision__ = '$Id$'

# Copyright © 2010 Michael Jahn
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published byp
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

import os
import logging
from urllib.request import urlcleanup
from gi.repository import (Gtk, GdkPixbuf)
import gutils
from edit import update_image_from_memory
from movie import Progress, Retriever
from plugins.extensions import GriffithExtensionBase as Base
from gettext import gettext as _

log = logging.getLogger('Griffith')


class GriffithExtension(Base):
    name = 'MoviePosterDB'
    description = _('Fetch posters from MoviePosterDB.com')
    author = 'Michael Jahn'
    email = 'mike@griffith.cc'
    version = 1
    api = 1
    enabled = True

    toolbar_icon = 'ge_movieposterdb.png'

    baseurltitle = 'http://www.movieposterdb.com/embed.inc.php?movie_title=%s'
    baseurltitleyear = 'http://www.movieposterdb.com/embed.inc.php?movie_title=%s[%s]'

    progress = None

    def toolbar_icon_clicked(self, widget, movie):
        log.info('fetching poster from MoviePosterDB.com...')
        self.movie = movie

        # correction of the movie name for the search
        o_title = None
        title = None
        if movie.o_title:
            if movie.o_title.endswith(', The'):
                o_title = "The %s" % movie.o_title[:-5]
            else:
                o_title = movie.o_title
        if movie.title:
            if movie.title.endswith(', The'):
                title = "The %s" % movie.title[:-5]
            else:
                title = movie.title

        # try to get an url to the large version of a poster for the movie
        # (requests are in the order:
        #  original title + year, original title only, title + year, title only)
        try:
            largeurl = None
            data = False
            if o_title:
                if movie.year:
                    url = self.baseurltitleyear % (o_title, movie.year)
                    data = self._get(url, self.widgets['window'])
                    if data and data.find('movie/0000000') < 0:
                        largeurl = gutils.trim(data, 'src=\\"', '\\"').replace('/t_', '/l_')
                if not data or not largeurl:
                    url = self.baseurltitle % o_title
                    data = self._get(url, self.widgets['window'])
                    if data and data.find('movie/0000000') < 0:
                        largeurl = gutils.trim(data, 'src=\\"', '\\"').replace('/t_', '/l_')
            if not data or not largeurl and title and title != o_title:
                if movie.year:
                    url = self.baseurltitleyear % (title, movie.year)
                    data = self._get(url, self.widgets['window'])
                    if data and data.find('movie/0000000') < 0:
                        largeurl = gutils.trim(data, 'src=\\"', '\\"').replace('/t_', '/l_')
                if not data or not largeurl:
                    url = self.baseurltitle % title
                    data = self._get(url, self.widgets['window'])
                    if data and data.find('movie/0000000') < 0:
                        largeurl = gutils.trim(data, 'src=\\"', '\\"').replace('/t_', '/l_')
        except Exception:
            log.exception('')
            gutils.warning(_('No posters found for this movie.'))
            return

        if not data or not largeurl:
            gutils.warning(_('No posters found for this movie.'))
            return

        # got the url for a large poster, fetch the data, show preview and update the
        # movie entry if the user want it
        data = self._get(largeurl, self.widgets['window'], decode=False)
        if not data:
            gutils.warning(_('No posters found for this movie.'))
            return

        if self._show_preview(data):
            update_image_from_memory(self.app, movie.number, data)

    def _show_preview(self, data):
        loader = GdkPixbuf.PixbufLoader()
        loader.write(data, len(data))
        loader.close()
        # show before set_from_pixbuf because it doesn't resize otherwise
        self.widgets['poster_window'].show()
        self.widgets['big_poster'].set_from_pixbuf(loader.get_pixbuf())
        result = gutils.question(_("Do you want to use this poster instead?"), self.widgets['window'])
        self.widgets['poster_window'].hide()
        return result

    def _get(self, url, parent_window, decode=True):
        # initialize the progress dialog once for the following search process
        if not self.progress:
            self.progress = Progress(parent_window)

        data = None
        url = url.encode('utf-8', 'replace')
        log.debug('Using url <%s>', url)
        self.progress.set_data(parent_window, _('Searching'), _('Wait a moment'), True)
        try:
            retriever = Retriever(url, parent_window, self.progress)
            retriever.start()
            while retriever.isAlive():
                self.progress.pulse()
                if self.progress.status:
                    retriever.join()
                while Gtk.events_pending():
                    Gtk.main_iteration()
            try:
                if retriever.html:
                    with open(retriever.html[0], 'rb') as ifile:
                        data = ifile.read()
                    # check for gzip compressed pages before decoding to unicode
                    if len(data) > 2 and data[0:2] == '\037\213':
                        data = gutils.decompress(data)
                    if decode:
                        data = data.decode('utf-8', 'replace')
                    os.remove(retriever.html[0])
            except IOError:
                log.exception('')
        finally:
            self.progress.hide()
        urlcleanup()
        return data



# -*- coding: utf-8 -*-

__revision__ = '$Id$'

# Copyright (c) 2009

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

import logging
import os
from gettext import gettext as _

import gutils
from plugins.imp import ImportPlugin as IP

log = logging.getLogger("Griffith")


class ImportPlugin(IP):
    description  = 'MovieTrack (v3.4x)'
    author       = 'Michael Jahn'
    email        = 'griffith@griffith.cc'
    version      = '1.0'
    file_filters = '*.[dD][aA][tT]'
    mime_types   = None

    fileversion  = None
    openedfile   = None
    imdbfilename = None

    def initialize(self):
        if not IP.initialize(self):
            return False
        self.edit = False
        return True

    def set_source(self, name):
        IP.set_source(self, name)
        self.filename = name
        self.fileversion = self.read_fileversion()
        if self.fileversion is None:
            gutils.error(_('The format of the file is not supported.'))
            return False
        if self.filename:
            self.imdbfilename = os.path.join(os.path.dirname(self.filename), 'IMDb.dat')
        else:
            self.imdbfilename = None
        return True

    def count_movies(self):
        """Returns number of movies in file which is about to be imported"""
        count = 0
        fileforcount = open(self.filename)
        try:
            for line in fileforcount:
                count = count + 1
        except Exception:
            log.exception('')
        finally:
            fileforcount.close()
        log.info('MovieTrack Import: %s movies for import' % count)
        return count

    def get_movie_details(self):
        """Returns dictionary with movie details"""
        if not self.openedfile:
            self.openedfile = open(self.filename)

        details = {}
        try:
            fields = []
            while len(fields) != 13:
                currentline = self.openedfile.readline()
                if not currentline:
                    return None
                fields = currentline[2:-2].split('||')

            if not fields[0] == 'NULL':
                details['title'] = str(fields[0])   # MovieName
            #details[''] = fields[1]    # SubEntry
            #details[''] = fields[2]    # Location
            if not fields[3] == 'NULL':
                details['genre'] = str(fields[3])   # Genre
            #details[''] = fields[4]    # Presence
            if not fields[5] == 'NULL':
                details['medium_id'] = fields[5]        # Medium
            if not fields[6] == 'NULL':
                try:
                    details['rating'] = int(gutils.before(fields[6], '/')) # Rating
                except Exception:
                    pass
            if not fields[7] == 'NULL':
                filepath = fields[7]                    # FilePath
                if not os.path.exists(filepath):
                    filename = os.path.basename(filepath)
                    filepath = os.path.join('Covers', filename)
                    if not os.path.exists(filepath):
                        filepath = filename
                        if not os.path.exists(filepath):
                            filepath = None
                if filepath:
                    details['poster'] = filepath
            if not fields[8] == 'NULL':
                details['barcode'] = str(fields[8]) # Barcode
            #details[''] = fields[9]    # AKA
            if not fields[10] == 'NULL':
                details['notes'] = str(fields[10])  # Comments
            movieid = fields[11]                        # AGN
            #details[''] = fields[12]   # UGN

            # try to get some more details from imdb-file
            try:
                if self.imdbfilename:
                    openedimdb = open(self.imdbfilename)
                    while fields:
                        fields = []
                        while len(fields) != 17:
                            currentline = openedimdb.readline()
                            if not currentline:
                                fields = None
                                break
                            fields = currentline[2:-2].split('||')
                        if fields and fields[1] == movieid:
                            #details[''] = fields[0]     # MovieName
                            #details[''] = fields[1]     # AGN
                            #details[''] = fields[2]     # Title
                            #details[''] = fields[3]     # AKA
                            if not fields[4] == 'NULL':
                                details['year'] = fields[4]              # Year
                            if not fields[5] == 'NULL':
                                details['director'] = str(fields[5]) # Director
                            #details[''] = fields[6]     # Tagline
                            if not fields[7] == 'NULL':
                                details['plot'] = str(fields[7])     # Summary
                            if not fields[8] == 'NULL' and 'genre' not in details:
                                details['genre'] = str(fields[8])   # Genre
                            if not fields[9] == 'NULL':
                                details['cast'] = str(fields[9])    # Actors
                            if not fields[10] == 'NULL':
                                details['classification'] = str(fields[10]) # MPAA
                            #details[''] = fields[11]    # Language
                            if not fields[12] == 'NULL':
                                details['runtime'] = fields[12]         # Runtime
                            if not fields[13] == 'NULL' and 'rating' not in details:
                                try:
                                    details['rating'] = int(gutils.before(fields[13], '/')) # Rating
                                except Exception:
                                    pass
                            #details[''] = fields[14]    # VoteCount
                            if not fields[15] == 'NULL':
                                details['site'] = str(fields[15])    # URL
                            #details[''] = fields[16]    # Exclude
            except Exception:
                log.exception('')
            finally:
                openedimdb.close()

        except EOFError:
            details = None
        except Exception:
            log.exception('')
            details = None

        return details

    def clear(self):
        """clear plugin before next source file"""
        IP.clear(self)
        if self.openedfile:
            self.openedfile.close()
            self.openedfile = None
        self.fileversion = None

    def destroy(self):
        """close all resources"""
        IP.destroy(self)

    def read_fileversion(self):
        version = None
        try:
            openfile = open(self.filename)
            try:
                firstline = openfile.readline()
                if firstline[0] == '~':
                    firstlineparts = firstline.split('||')
                    if len(firstlineparts) == 13:
                        version = 3.4
            finally:
                openfile.close()
        except Exception as e:
            log.error(str(e))
        log.info('MovieTrack Import: Found file version %s' % version)
        return version



# -*- coding: UTF-8 -*-
# vim: fdm=marker
__revision__ = '$Id$'

# Copyright © 2009 Piotr Ożarowski
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

import datetime
from sqlalchemy import Table, Column, ForeignKey, func
from sqlalchemy.types import Boolean, Unicode, Text, Integer, SmallInteger, \
    Date, LargeBinary, PickleType, DateTime

from db import metadata

posters = Table('posters', metadata,
                Column('md5sum', Unicode(32), primary_key=True),
                Column('data', LargeBinary(1048576), nullable=False))

volumes = Table('volumes', metadata,
                Column('volume_id', Integer, primary_key=True),
                Column('name', Unicode(64), nullable=False, unique=True),
                Column('loaned', Boolean, nullable=False, default=False))

collections = Table('collections', metadata,
                    Column('collection_id', Integer, primary_key=True),
                    Column('name', Unicode(64), nullable=False, unique=True),
                    Column('loaned', Boolean, nullable=False, default=False))

media = Table('media', metadata,
              Column('medium_id', Integer, primary_key=True),
              Column('name', Unicode(64), nullable=False, unique=True))

ratios = Table('ratios', metadata,
               Column('ratio_id', Integer, primary_key=True),
               Column('name', Unicode(5), nullable=False, unique=True))

vcodecs = Table('vcodecs', metadata,
                Column('vcodec_id', Integer, primary_key=True),
                Column('name', Unicode(64), nullable=False, unique=True))

movies = Table('movies', metadata,
               Column('movie_id', Integer, primary_key=True),
               Column('number', Integer, nullable=False, unique=True,
                      index=True),
               Column('collection_id', ForeignKey(collections.c.collection_id)),
               Column('volume_id', ForeignKey(volumes.c.volume_id)),
               Column('medium_id', ForeignKey(media.c.medium_id)),
               Column('ratio_id', ForeignKey(ratios.c.ratio_id)),
               Column('vcodec_id', ForeignKey(vcodecs.c.vcodec_id)),
               Column('poster_md5', ForeignKey(posters.c.md5sum)),
               Column('loaned', Boolean, nullable=False, default=False),
               Column('seen', Boolean, nullable=False, default=False),
               Column('rating', SmallInteger),
               Column('color', SmallInteger),
               Column('cond', SmallInteger),
               # MySQL will not accept name "condition"
               Column('layers', SmallInteger),
               Column('region', SmallInteger),
               Column('media_num', SmallInteger),
               Column('runtime', SmallInteger),
               Column('year', SmallInteger),
               Column('width', SmallInteger),
               Column('height', SmallInteger),
               Column('barcode', Unicode(32)),
               Column('o_title', Unicode(255), index=True),
               Column('title', Unicode(255), index=True),
               Column('director', Unicode(255)),
               Column('screenplay', Unicode(255)),
               Column('cameraman', Unicode(255)),
               Column('o_site', Unicode(255)),
               Column('site', Unicode(255)),
               Column('trailer', Unicode(255)),
               Column('country', Unicode(128)),
               Column('genre', Unicode(128)),
               Column('studio', Unicode(128)),
               Column('classification', Unicode(128)),
               Column('cast', Text),
               Column('plot', Text),
               Column('notes', Text),
               Column('image', Unicode(128)),  # XXX: deprecated
               Column('created', DateTime, default=datetime.datetime.now),
               Column('updated', DateTime, onupdate=datetime.datetime.now),
               )

people = Table('people', metadata,
               Column('person_id', Integer, primary_key=True),
               Column('name', Unicode(255), nullable=False, unique=True),
               Column('email', Unicode(128)),
               Column('phone', Unicode(64)))

loans = Table('loans', metadata,
              Column('loan_id', Integer, primary_key=True),
              Column('person_id', ForeignKey(people.c.person_id),
                     nullable=False),
              Column('movie_id', ForeignKey(movies.c.movie_id), nullable=False),
              Column('volume_id', ForeignKey(volumes.c.volume_id)),
              Column('collection_id', ForeignKey(collections.c.collection_id)),
              Column('date', Date, nullable=False, default=func.current_date()),
              Column('return_date', Date, nullable=True))

languages = Table('languages', metadata,
                  Column('lang_id', Integer, primary_key=True),
                  Column('name', Unicode(64), nullable=False, unique=True))

acodecs = Table('acodecs', metadata,
                Column('acodec_id', Integer, primary_key=True),
                Column('name', Unicode(64), nullable=False, unique=True))

achannels = Table('achannels', metadata,
                  Column('achannel_id', Integer, primary_key=True),
                  Column('name', Unicode(64), nullable=False, unique=True))

subformats = Table('subformats', metadata,
                   Column('subformat_id', Integer, primary_key=True),
                   Column('name', Unicode(64), nullable=False, unique=True))

tags = Table('tags', metadata,
             Column('tag_id', Integer, primary_key=True),
             Column('name', Unicode(64), nullable=False, unique=True))

movie_lang = Table('movie_lang', metadata,
                   Column('ml_id', Integer, primary_key=True),
                   Column('type', SmallInteger),
                   # 0: Original, 1:lector, 2:dubbing, 3:subtitle
                   Column('movie_id', ForeignKey(movies.c.movie_id),
                          nullable=False),
                   Column('lang_id', ForeignKey(languages.c.lang_id),
                          nullable=False),
                   Column('acodec_id', ForeignKey(acodecs.c.acodec_id)),
                   Column('achannel_id', ForeignKey(achannels.c.achannel_id)),
                   Column('subformat_id',
                          ForeignKey(subformats.c.subformat_id)))

movie_tag = Table('movie_tag', metadata,
                  Column('mt_id', Integer, primary_key=True),
                  Column('movie_id', ForeignKey(movies.c.movie_id)),
                  Column('tag_id', ForeignKey(tags.c.tag_id)))

configuration = Table('configuration', metadata,
                      Column('param', Unicode(16), primary_key=True),
                      Column('value', Unicode(128), nullable=False))

filters = Table('filters', metadata,
                Column('name', Unicode(64), primary_key=True),
                Column('data', PickleType, nullable=False))



# -*- coding: utf-8 -*-

__revision__ = '$Id$'

# Copyright (c) 2005-2009 Vasco Nunes, Piotr Ożarowski

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

import argparse
import logging
import os
import sys
from gettext import gettext as _
from locale import getlocale
from pathlib import Path

import gutils
# import should be here so that the default is set
from gdebug import griffith_debug

log = logging.getLogger("Griffith")


def define_command_line_arguments():
    parser = argparse.ArgumentParser(
        description="Griffith's command line arguments, all are optional")

    parser.add_argument('-v', '--version', action='store_true',
                        help="prints Griffith's version")
    parser.add_argument('-D', '--debug', action='store_true',
                        help="run with more debug info")
    parser.add_argument('-C', '--clean', action='store_true',
                        help="find and delete orphan files in posters "
                             "directory")
    parser.add_argument('--check-dep', action='store_true',
                        dest='check_dep',
                        help="check dependencies")
    parser.add_argument('--show-dep', action='store_true',
                        dest='show_dep', help="show dependencies")
    parser.add_argument('--shell', action='store_true',
                        help="open interactive shell")
    parser.add_argument('--sqlecho', action='store_true',
                        help="print SQL queries")
    parser.add_argument('--home', metavar='home_dir',
                        help="set Griffith's home directory (instead of the "
                             "default ~/.griffith")
    parser.add_argument('--config', metavar='ConfigFile',
                        help="an alternative Griffith configuration file")
    parser.add_argument('-c', '--cast', metavar='Cast',
                        help="Film cast")
    parser.add_argument('-d', '--director', metavar='Director',
                        help="director of the film")
    parser.add_argument('-o', '--original_title',
                        metavar='Original_Title',
                        help="Original title of the film")
    parser.add_argument('-t', '--title', metavar='Title',
                        help="Title of the film")
    parser.add_argument('-y', '--year', metavar='Year', type=int,
                        help="Release year of the film")
    parser.add_argument('--number', metavar='#Media',
                        type=int, help="Number of media")
    parser.add_argument('-s', '--sort', metavar='Sort_Order',
                        help="Specify sort order by listing the columns of "
                             "table 'Movie' comma separated")
    parser.add_argument('--seen', action='store_true',
                        help="has the film been watched")
    parser.add_argument('--loaned', action='store_true',
                        help="is the film loaned to somebody")
    parser.add_argument('--rating', metavar='Rating',
                        type=int, help="Rating of the film")
    return parser


def check_args():
    default_lang, default_enc = getlocale()
    if not default_enc:
        default_enc = 'UTF-8'

    config_filename = '.griffith.cfg'
    home = get_griffith_home_directory_path()

    parser = define_command_line_arguments()

    args = parser.parse_args()
    if args.version:
        import version
        print(version.pversion)
        sys.exit()
    elif args.debug:
        from platform import platform
        import version
        griffith_debug.set_debug(logdir=home)
        log.debug("Starting %s %s", version.pname, version.pversion)
        log.debug("Platform: %s (%s)", platform(), os.name)
        log.debug('Dependencies:')
        show_dependencies()
    elif args.clean:
        # FIXME: the function clean_posters_dir is called as a method but
        #  defined as a function
        gutils.clean_posters_dir()  # TODO: parameter 'self' unfilled on call
        sys.exit()
    elif args.sqlecho:
        sa_log = logging.getLogger("sqlalchemy")
        sa_log.setLevel(logging.INFO)
    elif args.home:
        home = args.home
    elif args.config:
        config_filename = args.config
    elif args.check_dep:
        check_dependencies()
        sys.exit()
    elif args.show_dep:
        show_dependencies()
        sys.exit()
    # set log file directory for current mode
    griffith_debug.set_logdir(home)
    return home, config_filename


def get_griffith_home_directory_path() -> str:
    home = Path.home() / ".griffith" / ""
    return str(home)


def check_args_with_db(griffith_obj):
    # FIXME: Why are there two command parsing functions? Potentially the
    #  checks on args.clean, args.debug, args.sort, args.shell should be only
    #  in check_args
    # FIXME: Originally all the command line arguments related to a film are
    #  in 'elif' clauses which likely doesn't make much sense. Therefore
    #  they have been made 'if' statements.
    if len(sys.argv) > 1:
        where = {}
        parser = define_command_line_arguments()
        args = parser.parse_args()
        sort = args.sort
        shell = args.shell
        if args.clean:
            gutils.clean_posters_dir(griffith_obj)
            sys.exit()
        elif args.debug:
            griffith_obj.debug_mode = True
        if args.original_title:
            where['o_title'] = args.original_title
        if args.title:
            where['title'] = args.title
        if args.director:
            where['director'] = args.director
        if args.cast:
            where['cast'] = args.cast
        if args.year:
            where['year'] = str(args.year)
        if args.seen:
            where['seen'] = args.seen
        if args.loaned:
            where['loaned'] = args.loaned
        if args.number:
            where['number'] = args.number
        # if args.runtime:
        #     where['runtime'] = args.runtime
        if args.rating:
            where['rating'] = args.rating
        if where:
            con_search_movie(griffith_obj, where, sort)
        if shell:
            run_shell(griffith_obj)


def con_search_movie(self, where, sort=None):
    # for search function
    from sqlalchemy import select
    import db
    mt = db.metadata.tables['movies'].columns
    columns = (mt.number, mt.title, mt.o_title, mt.director, mt.year)

    sort_columns = []
    if sort:
        for i in sort.split(','):
            if i in db.metadata.tables['movies'].columns:
                sort_columns.append(mt[i])
    else:
        sort_columns = [mt.number]

    statement = select(*columns).order_by(*sort_columns)

    for i in where:
        if i in ('seen', 'loaned'):    # force boolean
            if where[i].lower() in ('0', 'no', 'n'):
                where[i] = False
            else:
                where[i] = True
        if i in ('year', 'number', 'runtime', 'seen', 'loaned', 'rating'):
            statement = statement.where(mt[i] == where[i])
        else:
            statement = statement.where(mt[i].like('%' + where[i] + '%'))

    movies = self.db.session.execute(statement).fetchall()
    if not movies:
        print(_("No movie found"))
    else:
        for movie in movies:
            print("\033[31;1m[%s]\033[0m\t\033[38m%s\033[0m (\033[35m%s\033[0m)"
                  ", %s - \033[32m%s\033[0m" %
                  (movie.number, movie.title, movie.o_title, movie.year,
                   movie.director))
    sys.exit()


def check_dependencies():
    os_type = None
    if sys.version.rfind('Debian'):
        os_type = 'debian'

    (missing, extra) = gutils.get_dependencies()

    def __print_missing(modules):
        missing_modules = ''
        for i in modules:
            if i['version'] is False or (not isinstance(i['version'], bool)
                                         and i['version'].startswith('-')):
                tmp = None
                if os_type is not None:
                    if os_type == 'debian' and 'debian' in i:
                        tmp = "\n%s package" % i['debian']
                        if 'debian_req' in i and i['debian_req'] is not None:
                            tmp += ("\n\tminimum required package version: %s"
                                    % i['debian_req'])
                if tmp is None:
                    tmp = "\n%s module" % i['module']
                    if 'module_req' in i and i['module_req'] is not None:
                        tmp += ("\n\tminimum required module version: %s"
                                % i['module_req'])
                    if 'url' in i:
                        tmp += "\n\tURL: %s" % i['url']
                if i['version'] is not False and i['version'].startswith('-'):
                    tmp += "\n\tavailable module version: %s" % i['version'][1:]
                if tmp is not None:
                    missing_modules += tmp
        if not missing_modules:
            return None
        else:
            return missing_modules

    tmp = __print_missing(missing)
    if tmp:
        print('Dependencies missing:')
        print('====================')
        print(tmp)
    tmp = __print_missing(extra)
    if tmp:
        print('\n\nOptional dependencies missing:')
        print('=============================')
        print(tmp, "\n")


def show_dependencies():
    (missing, extra) = gutils.get_dependencies()
    for i in missing:
        print("%(module)s :: %(version)s" % i)
    for i in extra:
        print("%(module)s :: %(version)s" % i)


def run_shell(self):
    import sqlalchemy as sa
    from sqlalchemy.orm import sessionmaker
    import db

    # settings
    banner = """
Griffith Interactive Shell


Available variables:
* sess - database session (try f.e. `sess.bind.echo = True`)
* mem_sess - in memory session (playground)
* sa - SQLAlchemy module
* orm - SQLAlchemy's orm module
* db - ORM stuff (f.e. db.Movie class)
* gsql - GriffithSQL instance


Examples:
>>> movie = sess.query(db.Movie).first()
>>> print movie.title

>>> mem_movie = mem_sess.merge(movie)
>>> mem_movie.seen = False
>>> mem_sess.add(mem_movie)
>>> mem_sess.commit()

>>> seen_movies = sess.query(db.Movie).filter_by(seen=True)
>>> for movie in seen_movies:
        print(movie.title, movie.loaned)

>>> person = sess.query(db.Person).first()
>>> print("%s has %d movies loaned" % (person.name, person.loaned_movies_count))

>>> exit()
"""

    # prepare local environment ###

    # create a playground in memory
    mem_engine = sa.create_engine('sqlite:///:memory:')
    db.metadata.create_all(bind=mem_engine)  # create tables
    mem_session = sessionmaker(bind=mem_engine)
    mem_session = mem_session()

    locs = {
        'mem_sess': mem_session,
        'meta': db.metadata,
        'db': db,
        'sa': sa,
        'orm': sa.orm,
        'gsql': self.db,
        'sess': self.db.session,
        # 'griffith': self,
        }
    # exec 'from db import *' in locs

    # prepare the shell ###
    try:
        # FIXME: this whole block likely requires some review
        # try to use IPython if possible
        try:
            # >= 0.11
            from IPython.frontend.terminal.embed import InteractiveShellEmbed
            shell = InteractiveShellEmbed(banner2=banner)
        except ImportError:
            from IPython.frontend.terminal.embed import InteractiveShellEmbed
            # < 0.11
            from IPython.Shell import IPShellEmbed
            shell = IPShellEmbed(argv=self.args)
            shell.set_banner(shell.IP.BANNER + banner)
        shell(local_ns=locs, global_ns={})
    except ImportError:
        log.debug('IPython is not available')
        import code
        shell = code.InteractiveConsole(locals=locs)
        try:
            import readline
        except ImportError:
            log.debug('readline is not available')
        shell.interact(banner)
    sys.exit(0)



# -*- coding: utf-8 -*-

__revision__ = '$Id$'

# Copyright (c) 2009

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

import string
from gettext import gettext as _

import gutils
from plugins.imp import ImportPlugin as IP

try:
    import pyodbc
except ImportError:
    pass

import logging
log = logging.getLogger("Griffith")


class ImportPlugin(IP):
    description  = 'All My Movies (v5.7)'
    author       = 'Michael Jahn'
    email        = 'griffith@griffith.cc'
    version      = '1.0'
    file_filters = '*.[aA][mM][mM]'
    mime_types   = None

    fileversion  = None
    connection   = None
    cursor       = None

    def initialize(self):
        if not IP.initialize(self):
            return False
        self.edit = False
        return True

    def set_source(self, name):
        IP.set_source(self, name)
        self.filename = name
        self.fileversion = self.read_fileversion()
        if self.fileversion is None:
            gutils.error(_('The format of the file is not supported.'))
            return False
        return True

    def count_movies(self):
        """Returns number of movies in file which is about to be imported"""
        count = 0
        if self.connection:
            try:
                curs = self.connection.cursor()
                curs.execute('SELECT COUNT(*) FROM movies')
                count = curs.fetchone()[0]
            except Exception:
                log.exception('')
        else:
            log.error('AllMyMovies Import: No connection object.')
        log.info('AllMyMovies Import: %s movies for import' % count)
        return count

    def get_movie_details(self):
        """Returns dictionary with movie details"""
        if not self.cursor:
            self.cursor = self.connection.cursor()
            self.cursor.execute( \
                'SELECT MovieID, barcode, comments, description, length, mediacount, MediaTypeName, \
                        mpaa, movies.Name, originaltitle, rating, seen, studio, url, year, Trailer, \
                        MediaLocation.Name \
                 FROM (movies \
                      LEFT OUTER JOIN MediaType ON movies.MediaTypeID=MediaType.MediaTypeID) \
                      LEFT OUTER JOIN MediaLocation ON movies.medialocationID=MediaLocation.MediaLocationID;')

        currentrow = self.cursor.fetchone()
        if not currentrow:
            return None

        details = {}
        try:
            movieID = currentrow[0]
            if currentrow[1]:
                details['barcode'] = currentrow[1]
            if currentrow[2]:
                details['notes'] = currentrow[2]
            if currentrow[3]:
                details['plot'] = currentrow[3]
            if currentrow[4]:
                details['runtime'] = gutils.digits_only(currentrow[4])
            if currentrow[5]:
                details['media_num'] = currentrow[5]
            if currentrow[6]:
                details['medium_id'] = currentrow[6]
            if currentrow[7]:
                details['classification'] = currentrow[7]
            if currentrow[8]:
                details['title'] = currentrow[8]
            if currentrow[9]:
                details['o_title'] = currentrow[9]
            if currentrow[10]:
                details['rating'] = round(currentrow[10] / 10.0, 0)
            if currentrow[11]:
                details['seen'] = currentrow[11]
            if currentrow[12]:
                details['studio'] = currentrow[12]
            if currentrow[13]:
                details['site'] = currentrow[13]
            if currentrow[14]:
                details['year'] = currentrow[14]
            if currentrow[15]:
                details['trailer'] = currentrow[15]
            if currentrow[16]:
                details['volume_id'] = currentrow[16]
            # loading other details
            curs = self.connection.cursor()
            curs.execute('SELECT Actors.Name, ActorsLink.Role \
                          FROM Actors INNER JOIN ActorsLink ON Actors.ActorID=ActorsLink.ActorID \
                          WHERE ActorsLink.MovieID=' + str(movieID))
            currentrow = curs.fetchone()
            if currentrow:
                if currentrow[1]:
                    cast = currentrow[0] + _(' as ') + currentrow[1]
                else:
                    cast = currentrow[0]
                currentrow = curs.fetchone()
                while currentrow:
                    if currentrow[1]:
                        cast = cast + '\n' + currentrow[0] + _(' as ') + currentrow[1]
                    else:
                        cast = cast + '\n' + currentrow[0]
                    currentrow = curs.fetchone()
                details['cast'] = cast

            curs.execute('SELECT Countries.Name \
                          FROM Countries INNER JOIN CountryLink ON Countries.CountryID=CountryLink.CountryID \
                          WHERE CountryLink.MovieID=' + str(movieID))
            currentrow = curs.fetchone()
            if currentrow:
                country = currentrow[0]
                currentrow = curs.fetchone()
                while currentrow:
                    country = country + ', ' + currentrow[0]
                    currentrow = curs.fetchone()
                details['country'] = country

            curs.execute('SELECT Actors.Name \
                          FROM Actors INNER JOIN DirectorLink ON Actors.ActorID=DirectorLink.ActorID \
                          WHERE DirectorLink.MovieID=' + str(movieID))
            currentrow = curs.fetchone()
            if currentrow:
                director = currentrow[0]
                currentrow = curs.fetchone()
                while currentrow:
                    director = director + ', ' + currentrow[0]
                    currentrow = curs.fetchone()
                details['director'] = director

            curs.execute('SELECT Genres.Name \
                          FROM Genres INNER JOIN GenresLink ON Genres.GenreID=GenresLink.GenreID \
                          WHERE GenresLink.MovieID=' + str(movieID))
            currentrow = curs.fetchone()
            if currentrow:
                genre = currentrow[0]
                currentrow = curs.fetchone()
                while currentrow:
                    genre = genre + ', ' + currentrow[0]
                    currentrow = curs.fetchone()
                details['genre'] = genre

            curs.execute('SELECT images.image \
                          FROM images \
                          WHERE cover=-1 AND images.MovieID=' + str(movieID))
            currentrow = curs.fetchone()
            if currentrow:
                details['poster'] = currentrow[0]

            curs.close()
        except Exception:
            log.exception('')
            details = None
        #print details
        return details

    def clear(self):
        """clear plugin before next source file"""
        IP.clear(self)
        if self.cursor:
            self.cursor.close()
            self.cursor = None
        if self.connection:
            self.connection.close()
            self.connection = None
        self.fileversion = None

    def destroy(self):
        """close all resources"""
        IP.destroy(self)

    def read_fileversion(self):
        version = None
        try:
            log.debug('Connecting with: DRIVER={Microsoft Access Driver (*.mdb)};DBQ=%s' % string.replace(self.filename, '\\', '\\\\'))
            self.connection = pyodbc.connect('DRIVER={Microsoft Access Driver (*.mdb)};DBQ=%s' % string.replace(self.filename, '\\', '\\\\'))
            curs = self.connection.cursor()
            tablerow = curs.tables(table='movies').fetchone()
            if tablerow:
                version = 5.7
            curs.close()
        except Exception as e:
            log.error(str(e))
        log.info('AllMyMovies Import: Found file version %s' % version)
        return version



# -*- coding: utf-8 -*-

__revision__ = '$Id$'

# Copyright (c) 2009

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

from plugins.imp import ImportPlugin as IP
import os
import gutils
from xml.dom import minidom
from gettext import gettext as _

import logging
log = logging.getLogger("Griffith")


class ImportPlugin(IP):
    description  = 'GCstar (v1.4.x)'
    author       = 'Michael Jahn'
    email        = 'griffith@griffith.cc'
    version      = '1.0'
    file_filters = '*.[gG][cC][sS]'
    mime_types   = None

    fileversion  = None
    filedom      = None
    items        = None
    itemindex    = 0

    def initialize(self):
        if not IP.initialize(self):
            return False
        self.edit = False
        return True

    def set_source(self, name):
        IP.set_source(self, name)
        self.filename = name
        self.fileversion = self.read_fileversion()
        if self.fileversion is None:
            gutils.error(_('The format of the file is not supported.'))
            return False
        return True

    def count_movies(self):
        """Returns number of movies in file which is about to be imported"""
        count = 0
        if self.filedom:
            try:
                collectionElement = self.filedom.getElementsByTagName('collection')[0]
                count = int(collectionElement.getAttribute('items'))
            except Exception as e:
                log.error(str(e))
        else:
            log.error('No filedom object.')
        return count

    def get_movie_details(self):
        """Returns dictionary with movie details"""
        if not self.filedom:
            self.filedom = minidom.parse(self.filename)
        if not self.items:
            self.items = self.filedom.getElementsByTagName('item')
            self.itemindex = 0
        if not self.items or len(self.items) < 1:
            return None
        if len(self.items) <= self.itemindex:
            return None

        details = {}
        try:
            item = self.items[self.itemindex]

            details['title']          = item.getAttribute('title')
            details['o_title']        = item.getAttribute('original')
            details['year']           = item.getAttribute('date')
            details['runtime']        = item.getAttribute('time')
            details['country']        = item.getAttribute('country')
            details['site']           = item.getAttribute('webPage')
            details['seen']           = item.getAttribute('seen') == '1'
            details['director']       = item.getAttribute('director')
            # image filenames are mostly a absolute path at the original system
            # which should be now a relative path to the xml database file
            imagefilename             = item.getAttribute('image')
            previoustail = ''
            previoushead = imagefilename
            while not os.path.exists(imagefilename) and previoushead:
                (head, tail) = os.path.split(previoushead)
                imagefilename = os.path.join(os.path.join(os.path.dirname(self.filename), tail), previoustail)
                previoustail = tail
                previoushead = head
            if os.path.exists(imagefilename):
                details['poster'] = imagefilename
            try:
                details['number']     = int(item.getAttribute('id'))
            except Exception:
                pass
            try:
                details['media_num']  = int(item.getAttribute('number'))
            except Exception:
                pass
            details['trailer']        = item.getAttribute('trailer')
            details['classification'] = item.getAttribute('age')
            details['rating']         = item.getAttribute('rating')
            synopsis = item.getElementsByTagName('synopsis')
            if len(synopsis) and len(synopsis[0].childNodes):
                details['plot'] = synopsis[0].childNodes[0].data
            comment = item.getElementsByTagName('comment')
            if len(comment) and len(comment[0].childNodes):
                details['notes'] = comment[0].childNodes[0].data
            genres = item.getElementsByTagName('genre')
            if len(genres):
                genrelines = genres[0].getElementsByTagName('line')
                genre = ''
                try:
                    for genreline in genrelines:
                        genrecols = genreline.getElementsByTagName('col')
                        if len(genrecols) and len(genrecols[0].childNodes):
                            genre = genre + genrecols[0].childNodes[0].data + ', '
                except Exception as ex:
                    log.error(str(ex))
                if len(genre) > 2:
                    details['genre'] = genre[:-2]
            actors = item.getElementsByTagName('actors')
            if len(actors):
                actorlines = actors[0].getElementsByTagName('line')
                cast = ''
                try:
                    for actorline in actorlines:
                        actorcols = actorline.getElementsByTagName('col')
                        if len(actorcols):
                            if len(actorcols) > 1 and len(actorcols[0].childNodes) and len(actorcols[1].childNodes):
                                cast = cast + actorcols[0].childNodes[0].data + _(' as ') + actorcols[1].childNodes[0].data + "\n"
                            else:
                                if len(actorcols[0].childNodes):
                                    cast = cast + actorcols[0].childNodes[0].data + "\n"
                except Exception as ex:
                    log.error(str(ex))
                if len(cast):
                    details['cast'] = cast
        except EOFError:
            details = None
        except Exception as e:
            log.error(str(e))
            details = None
        self.itemindex = self.itemindex + 1
        return details

    def clear(self):
        """clear plugin before next source file"""
        IP.clear(self)
        if self.filedom:
            self.filedom.unlink()
            self.filedom = None
            self.fileversion = None
            self.items = None
            self.itemindex = 0

    def destroy(self):
        """close all resources"""
        IP.destroy(self)

    def read_fileversion(self):
        version = None
        self.filedom = minidom.parse(self.filename)
        try:
            collectionElement = self.filedom.getElementsByTagName('collection')[0]
            type = collectionElement.getAttribute('type')
            if type == 'GCfilms':
                version = collectionElement.getAttribute('version')
        except Exception as e:
            log.error(str(e))
            self.filedom.unlink()
            self.filedom = None
        return version



# -*- coding: UTF-8 -*-

__revision__ = '$Id$'

# Copyright (c) 2005-2009 Vasco Nunes, Piotr Ożarowski
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Library General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

# You may use and distribute this software under the terms of the
# GNU General Public License, version 2 or later

from gettext import gettext as _
import locale
import logging
import os
import tempfile
import threading
import time
import urllib.error
import urllib.parse
import urllib.request
from urllib.parse import quote
from urllib.request import urlcleanup, FancyURLopener

from gi.repository import Gtk
import gutils

log = logging.getLogger("Griffith")


class Movie(object):
    cast = ''
    classification = ''
    country = ''
    director = ''
    genre = ''
    image = ''
    notes = ''
    number = None
    o_site = ''
    o_title = ''
    plot = ''
    rating = ''
    runtime = ''
    site = ''
    studio = ''
    title = ''
    trailer = ''
    year = ''
    screenplay = ''
    cameraman = ''
    resolution = ''
    barcode = ''

    movie_id = ''
    debug = False
    locations = ''
    engine_author = None
    engine_description = None
    engine_language = None
    engine_name = None
    engine_version = None
    page = None
    url = ''
    image_url = ''
    encode = 'utf-8'
    fields_to_fetch = []
    progress = None
    useurllib2 = False
    parent_window = None  # had been defined outside __init__

    # functions that plugin should implement: {{{
    def initialize(self):
        pass

    def get_barcode(self):
        pass

    def get_cameraman(self):
        pass

    def get_cast(self):
        pass

    def get_classification(self):
        pass

    def get_country(self):
        pass

    def get_director(self):
        pass

    def get_genre(self):
        pass

    def get_image(self):
        pass

    def get_notes(self):
        pass

    def get_o_site(self):
        pass

    def get_o_title(self):
        pass

    def get_plot(self):
        pass

    def get_rating(self):
        pass

    def get_resolution(self):
        pass

    def get_runtime(self):
        pass

    def get_screenplay(self):
        pass

    def get_site(self):
        pass

    def get_studio(self):
        pass

    def get_title(self):
        pass

    def get_trailer(self):
        pass

    def get_year(self):
        pass
    # }}}

    def __getitem__(self, key):
        return getattr(self, key)

    def __setitem__(self, key, value):
        setattr(self, key, value)

    def get_movie(self, parent_window=None):
        try:
            #
            # initialize the progress dialog once for the following loading process
            #
            if self.progress is None:
                self.progress = Progress(parent_window)
            self.progress.set_data(
                parent_window, _("Checking connection"), _("Wait a moment"),
                True)
            #
            # check for internet connection
            urllib.request.urlopen("https://www.google.com")
            self.progress.set_data(
                parent_window, _("Fetching data"), _("Wait a moment"), True)
            #
            # get the page
            #
            if not self.open_page(parent_window):
                return None
            return True

        except Exception as exc:
            log.error(f"[E017]Caught too broad try-except,"
                      f" {type(exc).__name__}: {exc}")
            self.progress.hide()
            gutils.error(_("Connection failed."))
            return False

    def open_page(self, parent_window=None, url=None):
        log.debug("[open_page]")
        if url is None:
            url_to_fetch = self.url
        else:
            url_to_fetch = url
        if parent_window is not None:
            self.parent_window = parent_window
        self.progress.set_data(
            parent_window, _("Fetching data"), _("Wait a moment"), False)
        retriever = Retriever(
            url_to_fetch, self.parent_window, self.progress,
            useurllib2=self.useurllib2)
        retriever.start()
        log.debug(f"self.progress.status: retriever.start(), url: {self.url}")
        while retriever.is_alive():
            self.progress.pulse()
            if self.progress.status:
                retriever.join()
            while Gtk.events_pending():
                Gtk.main_iteration()
        data = None
        try:
            if retriever.exception is None:
                if retriever.html:
                    log.debug(f"HTML retrieval: {retriever.html}")
                    ifile = open(retriever.html[0], "r", encoding=self.encode, errors='replace')
                    try:
                        data = ifile.read()
                    finally:
                        ifile.close()
                    # check for gzip compressed pages before decoding to unicode
                    if len(data) > 2 and data[0:2] == '\037\213':
                        data = gutils.decompress(data)
            else:
                log.warning(f"Unable to retrieve: {retriever.exception}")
                gutils.urllib_error(_("Connection error"), self.parent_window)
        except IOError as exc:
            log.exception(f"IOError: {exc}")
        log.debug(f"Retrieved data size: {len(data)}")
        if url is None:
            self.page = data
        urlcleanup()
        return data

    def fetch_picture(self):
        if self.image_url:
            tmp_dest = tempfile.mktemp(prefix='poster_', dir=self.locations['temp'])
            log.debug(f"fetch_picture: tmp_dest: {tmp_dest}")
            self.image = tmp_dest.split('poster_', 1)[1]
            log.debug(f"fetch_picture: self.image: {self.image}")
            dest = "%s.jpg" % tmp_dest
            try:
                self.progress.set_data(
                    self.parent_window,
                    _("Fetching poster"),
                    _("Wait a moment"),
                    False)
                retriever = Retriever(self.image_url, self.parent_window,
                                      self.progress, dest,
                                      useurllib2=self.useurllib2)
                retriever.start()
                while retriever.is_alive():
                    self.progress.pulse()
                    if self.progress.status:
                        retriever.join()
                    while Gtk.events_pending():
                        Gtk.main_iteration()
                urlcleanup()
            except Exception as exc:
                log.exception(f"[E018]Caught too broad try-except,"
                              f" {type(exc).__name__}: {exc}")
                self.image = ""
                try:
                    os.remove("%s.jpg" % tmp_dest)
                except Exception as exc:
                    log.info(f"Can't remove {tmp_dest} file, {exc}")
        else:
            self.image = ""

    def parse_movie(self):
        log.debug("[parse_movie]")
        try:
            fields = list(self.fields_to_fetch)  # make a copy

            self.initialize()

            if 'year' in fields:
                self.get_year()
                self.year = gutils.digits_only(self.year, 2100)
                fields.pop(fields.index('year'))
            if 'runtime' in fields:
                self.get_runtime()
                self.runtime = gutils.digits_only(self.runtime)
                fields.pop(fields.index('runtime'))
            if 'rating' in fields:
                self.get_rating()
                self.rating = gutils.digits_only(self.rating, 10)
                fields.pop(fields.index('rating'))
            if 'cast' in fields:
                self.get_cast()
                self.cast = gutils.clean(self.cast)
                if not isinstance(self.cast, str):
                    self.cast = gutils.gdecode(self.cast, self.encode)
                fields.pop(fields.index('cast'))
            if 'plot' in fields:
                self.get_plot()
                log.debug(f"self.plot: {self.plot}")
                self.plot = gutils.clean(self.plot)
                if not isinstance(self.plot, str):
                    self.plot = gutils.gdecode(self.plot, self.encode)
                fields.pop(fields.index('plot'))
            if 'notes' in fields:
                self.get_notes()
                self.notes = gutils.clean(self.notes)
                if not isinstance(self.notes, str):
                    self.notes = gutils.gdecode(self.notes, self.encode)
                fields.pop(fields.index('notes'))
            if 'image' in fields:
                log.debug(f"image in fields: {fields}")
                self.get_image()
                log.debug(f"self.image_url: {self.image_url} self.image: {self.image}")
                self.fetch_picture()
                fields.pop(fields.index('image'))

            for i in fields:
                getattr(self, "get_%s" % i)()
                self[i] = gutils.clean(self[i])
                if not isinstance(self[i], str):
                    self[i] = gutils.gdecode(self[i], self.encode)

            if 'o_title' in self.fields_to_fetch and self.o_title is not None:
                if self.o_title[:4] == 'The ':
                    self.o_title = self.o_title[4:] + ', The'
            if 'title' in self.fields_to_fetch and self.title is not None:
                if self.title[:4] == 'The ':
                    self.title = self.title[4:] + ', The'
        except Exception as exc:
            log.exception(f"{exc}")
        finally:
            # close the progress dialog which was opened in get_movie
            self.progress.hide()


class SearchMovie(object):
    page = None
    number_results = None
    titles = []
    ids = []
    url = None
    encode = 'utf-8'
    original_url_search = None
    translated_url_search = None
    elements = None
    title = None
    remove_accents = True
    progress = None
    useurllib2 = False
    usepostrequest = False

    def __init__(self):
        pass

    def search_movies(self, parent_window):
        try:
            #
            # initialize the progress dialog once for the following search process
            #
            if self.progress is None:
                self.progress = Progress(parent_window)
            self.progress.set_data(parent_window, _("Searching"), _("Wait a moment"), True)
            #
            # call the plugin specific search method
            #
            return self.search(parent_window)
        finally:
            # don't forget to hide the progress dialog
            self.progress.hide()

    def open_search(self, parent_window, destination=None):
        log.debug("[open_search]")
        self.titles = [""]
        self.ids = [""]

        self.title = quote(self.title)
        if self.url.find('%s') > 0:
            self.url %= self.title
            self.url = self.url.replace(' ', '%20')
        else:
            if not self.usepostrequest:
                tempstr = self.url + self.title
                self.url = tempstr.replace(' ', '%20')
        url = self.url
        self.progress.set_data(parent_window, _("Searching"), _("Wait a moment"), True)
        if self.usepostrequest:
            postdata = self.get_postdata()
            retriever = Retriever(
                url, parent_window, self.progress, destination,
                useurllib2=self.useurllib2, postdata=postdata)
        else:
            retriever = Retriever(url, parent_window, self.progress,
                                  destination, useurllib2=self.useurllib2)
        retriever.start()
        while retriever.is_alive():
            self.progress.pulse()
            if self.progress.status:
                retriever.join()
            while Gtk.events_pending():
                Gtk.main_iteration()
        try:
            if retriever.exception is None:
                if destination:
                    # caller gave an explicit destination file
                    # don't care about the content
                    return True
                if retriever.html:
                    with open(retriever.html[0], 'rb') as ifile:
                        self.page = ifile.read()
                    # check for gzip compressed pages before decoding to unicode
                    if len(self.page) > 2 and self.page[0:2] == '\037\213':
                        self.page = gutils.decompress(self.page)
                    self.page = self.page.decode(self.encode, 'replace')
                else:
                    return False
            else:
                self.progress.hide()
                gutils.urllib_error(_("Connection error"), parent_window)
                return False
        except IOError as exc:
            log.exception(f"IOError: {exc}")
        finally:
            urlcleanup()
        return True

    def get_postdata(self):
        # sample, depends on target site
        # return {'title' : self.Title, 'category' : 'movieTitel' }
        return {}


class Retriever(threading.Thread):

    def __init__(self, url, parent_window, progress, destination=None, useurllib2=False, postdata=None):
        self.URL = url
        self.html = None
        self.exception = None
        self.destination = destination
        self.useurllib2 = useurllib2
        self.progress = progress
        self._stopevent = threading.Event()
        self._sleepperiod = 1.0
        self.postdata = postdata
        threading.Thread.__init__(self, name="Retriever")

    def run(self):
        try:
            if self.useurllib2:
                if self.postdata:
                    encodedpostdata = urllib.parse.urlencode(self.postdata)
                    self.html = urlretrieve2(self.URL, self.destination,
                                             self.hook, encodedpostdata)
                    log.debug(f"urllib2 POST HTML=\n{self.html}")
                else:
                    self.html = urlretrieve2(self.URL, self.destination,
                                             self.hook)
            else:
                if self.postdata:
                    encodedpostdata = urllib.parse.urlencode(self.postdata)
                    log.debug(f"encodedpostdata={encodedpostdata}")
                    self.html = urlretrieve(self.URL, self.destination,
                                            self.hook, encodedpostdata)
                    log.debug(f"urllib POST data={self.postdata} "
                              f"HTML=\n{self.html}")
                else:
                    try:
                        log.debug("No encodedpostdata")
                        self.html = urlretrieve(self.URL, self.destination,
                                                self.hook)
                        log.debug(f"urllib POST HTML=\n{self.html}")
                    except ValueError as exc:
                        msg = f"Could not retrieve data from URL {self.URL}"
                        log.debug(f"No exception 'ValueError', "
                                  f"self.html={self.html}")
                        log.error(f"{msg}, {exc}")
                        gutils.error(_(msg))
            if self.progress.status:
                log.debug(f"self.progress.status={self.progress.status}")
                self.html = []
        except Exception as exc:
            log.exception(f"[E019]Caught too broad try-except,"
                          f" {type(exc).__name__}: {exc}")
            self.exception = exc

    def hook(self, count, block_size, total_size):
        if total_size == -1:
            pass
        else:
            try:
                min((count * block_size * 100) /
                                            total_size, 100)
            except Exception:
                pass
            if count != 0:
                int(count * block_size / 1024.0)
                int(total_size / 1024.0)

#
# use own derived URLopener class because we need to set a correct User-Agent
# string for some websites. The default is 'Python-urllib/<version>'
# which not all sites accepting. (zelluloid.de for example)
#


_uaurlopener = None
_tempfilecleanup = None


def urlretrieve(url, filename=None, reporthook=None, data=None):
    global _uaurlopener
    if not _uaurlopener:
        _uaurlopener = UAFancyURLopener()
    log.debug(f"url='{url}' _uaurlopener={_uaurlopener}")
    if _uaurlopener and url:
        try:
            ua_url_opener = _uaurlopener.retrieve(url, filename, reporthook,
                                                  data)
        except (AttributeError, OSError) as exc:
            msg = f"Retrieving URL={url}. filename={filename} failed. exc={exc}"
            log.error(msg)
            gutils.error(msg)
            ua_url_opener = None
    else:
        return None
    if ua_url_opener:
        return _uaurlopener.retrieve(url, filename, reporthook, data)
    else:
        return None


def urlretrieve2(url, filename=None, reporthook=None, data=None):
    global _tempfilecleanup
    headers = {
        # 'User-Agent': 'Mozilla/5.0 (compatible; MSIE 10.0; Windows NT 6.2; Trident/6.0; MSAppHost/1.0)',
        # 'User-Agent': 'Dalvik/1.6.0 (Linux; U; Android 4.2.2; Nexus 4 Build/JDQ39E)'
        'User-Agent': 'Dalvik/1.2.0 (Linux; U; Android 2.2.2; Huawei U8800-51 Build/HWU8800B635)',
        'Cache-Control': 'no-cache',
        'Pragma': 'no-cache',
        'Accept-Encoding': 'gzip'}
    req = urllib.request.Request(url, data, headers)
    response = urllib.request.urlopen(req)
    if not filename:
        import tempfile
        (fd, filename) = tempfile.mkstemp()
        if not _tempfilecleanup:
            _tempfilecleanup = TempFileCleanup()
        _tempfilecleanup._tempfiles.append(filename)
    else:
        fd = filename
    with open(fd, 'wb') as tfp:
        while 1:
            block = response.read(4096)
            if block == "":
                break
            tfp.write(block)
    return [filename, response.info()]


def get_locale():
    os_locale = locale.getdefaultlocale()[0]
    if os_locale is None:
        return "en-US"
    return os_locale.replace('_', '-')


class UAFancyURLopener(FancyURLopener):
    # use Firefox 3.0 User-Agent string from Windows XP
    version = 'Mozilla/5.0 (Windows; U; Windows NT 6.0) Gecko/2008052906 Firefox/3.0'

    def __init__(self, *args, **kwargs):
        FancyURLopener.__init__(self, *args, **kwargs)
        # additional HTTP headers to work around the html file compression which
        # is used by UMTS connections. compression breaks the movie import plugins sometimes
        self.addheader('Cache-Control', 'no-cache')
        self.addheader('Pragma', 'no-cache')
        self.addheader('Accept-Language', '%s,en;q=0.5' % get_locale())

    def open(self, fullurl, data=None):
        # prevent blocking calls which doesn't come back
        import socket
        socket.setdefaulttimeout(20)
        try:
            fancy_url_opener_fp = FancyURLopener.open(self, fullurl, data)
        except urllib.error.URLError as exc:
            log.error(f"Can't open URL {fullurl}, exception: {exc}")
            fancy_url_opener_fp = None
        return fancy_url_opener_fp


class TempFileCleanup:
    _tempfiles = []

    def __init__(self):
        self.__unlink = os.unlink

    def __del__(self):
        self.cleanup()

    def cleanup(self):
        if self._tempfiles:
            for file in self._tempfiles:
                try:
                    self.__unlink(file)
                except OSError:
                    pass
            del self._tempfiles[:]


class Progress:

    def __init__(self, window, title='', message=''):
        self.status = False
        self.dialog = Gtk.Dialog(title, window, Gtk.DialogFlags.MODAL, ())
        self.dialog.set_urgency_hint(False)
        self.label = Gtk.Label()
        self.label.set_markup(message)
        self.dialog.vbox.pack_start(self.label, True, True, 0)
        self.progress = Gtk.ProgressBar()
        self.progress.set_pulse_step(0.01)
        self.dialog.vbox.pack_start(self.progress, False, False, 0)
        self.button = Gtk.Button(_("Cancel"), Gtk.STOCK_CANCEL)
        self.button.connect("clicked", self.callback)
        self.dialog.vbox.pack_start(self.button, False, False, 0)
        self.dialog.show_all()

    def callback(self, widget):
        self.dialog.hide()
        self.status = True

    def pulse(self):
        self.progress.pulse()
        time.sleep(0.01)

    def close(self):
        self.dialog.destroy()

    def hide(self):
        self.dialog.hide()

    def set_data(self, parent_window, title, message, showit):
        # self.dialog.set_parent(parent_window)
        self.dialog.set_title(title)
        self.label.set_markup(message)
        if showit is True:
            self.dialog.show()
# vim: fdm=marker
